Package: GenomicSuperSignature
Title: Interpretation of RNA-seq experiments through robust, efficient
        comparison to public databases
Authors@R: c(
  person("Sehyun", "Oh", email = "shbrief@gmail.com", role = c("aut","cre")),
  person("Levi", "Waldron", role = "aut"),
  person("Sean", "Davis", email = "seandavi@gmail.com", role = "aut"))
Version: 1.18.0
Date: 2025-9-30
Description: This package provides a novel method for interpreting new 
  transcriptomic datasets through near-instantaneous comparison to public 
  archives without high-performance computing requirements. Through the 
  pre-computed index, users can identify public resources associated with
  their dataset such as gene sets, MeSH term, and publication. Functions to 
  identify interpretable annotations and intuitive visualization options are 
  implemented in this package. 
Depends: R (>= 4.1.0), SummarizedExperiment
Imports: ComplexHeatmap, ggplot2, methods, S4Vectors, Biobase, ggpubr,
        dplyr, plotly, BiocFileCache, grid, flextable, irlba
Suggests: knitr, rmarkdown, devtools, roxygen2, pkgdown, usethis,
        BiocStyle, testthat, forcats, stats, wordcloud, circlize,
        EnrichmentBrowser, clusterProfiler, msigdbr, cluster,
        RColorBrewer, reshape2, tibble, BiocManager, bcellViper, readr,
        utils
License: Artistic-2.0
biocViews: Transcriptomics, SystemsBiology, PrincipalComponent, RNASeq,
        Sequencing, Pathways, Clustering
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
VignetteBuilder: knitr
URL: https://github.com/shbrief/GenomicSuperSignature
BugReports: https://github.com/shbrief/GenomicSuperSignature/issues
Collate: 'GenomicSignatures-class.R' 'GenomicSignatures-methods.R'
        'PCAGenomicSignatures-methods.R' 'annotatePC.R' 'annotateRAV.R'
        'availableRAVmodel.R' 'buildAvgLoading.R' 'calculateScore.R'
        'data.R' 'drawWordcloud.R' 'extractPC.R' 'findSignature.R'
        'findStudiesInCluster.R' 'getMetadata.R' 'getModel.R'
        'heatmapTable.R' 'plotAnnotatedPCA.R' 'plotValidate.R'
        'rmNaInf.R' 'sampleScoreHeatmap.R' 'subsetEnrichedPathways.R'
        'utils.R' 'validate.R' 'validatedSignatures.R'
git_url: https://git.bioconductor.org/packages/GenomicSuperSignature
git_branch: RELEASE_3_22
git_last_commit: e2b134e
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 04:11:38 UTC; biocbuild
Author: Sehyun Oh [aut, cre],
  Levi Waldron [aut],
  Sean Davis [aut]
Maintainer: Sehyun Oh <shbrief@gmail.com>

# Generated by roxygen2: do not edit by hand

export("PCAsummary<-")
export("geneSets<-")
export("gsea<-")
export("mesh<-")
export("silhouetteWidth<-")
export("studies<-")
export("trainingData<-")
export("updateNote<-")
export(PCAGenomicSignatures)
export(PCAsummary)
export(PCinRAV)
export(RAVindex)
export(annotatePC)
export(annotateRAV)
export(availableRAVmodel)
export(buildAvgLoading)
export(calculateScore)
export(drawWordcloud)
export(extractPC)
export(findKeywordInRAV)
export(findSignature)
export(findStudiesInCluster)
export(geneSets)
export(getModel)
export(getRAVInfo)
export(getStudyInfo)
export(gsea)
export(heatmapTable)
export(mesh)
export(meshTable)
export(plotAnnotatedPCA)
export(plotValidate)
export(rmNaInf)
export(sampleScoreHeatmap)
export(silhouetteWidth)
export(studies)
export(subsetEnrichedPathways)
export(trainingData)
export(updateNote)
export(validate)
export(validatedSignatures)
export(version)
exportClasses(GenomicSignatures)
exportClasses(PCAGenomicSignatures)
exportMethods("PCAsummary<-")
exportMethods("geneSets<-")
exportMethods("gsea<-")
exportMethods("mesh<-")
exportMethods("silhouetteWidth<-")
exportMethods("studies<-")
exportMethods("trainingData<-")
exportMethods("updateNote<-")
exportMethods(PCAsummary)
exportMethods(RAVindex)
exportMethods(geneSets)
exportMethods(gsea)
exportMethods(mesh)
exportMethods(show)
exportMethods(silhouetteWidth)
exportMethods(studies)
exportMethods(trainingData)
exportMethods(updateNote)
exportMethods(version)
import(ComplexHeatmap)
import(SummarizedExperiment)
import(dplyr)
import(ggplot2)
import(ggpubr)
import(methods)
importFrom(BiocFileCache,bfcadd)
importFrom(BiocFileCache,bfcdownload)
importFrom(BiocFileCache,bfcneedsupdate)
importFrom(BiocFileCache,bfcquery)
importFrom(BiocFileCache,bfcrpath)
importFrom(S4Vectors,metadata)
importFrom(grid,gpar)
importFrom(irlba,irlba)

# GenomicSuperSignature
**Interpretation of RNA-seq experiments through robust, efficient comparison to public databases**


### Purpose
Thousands of RNA sequencing profiles have been deposited in public archives, yet 
remain unused for the interpretation of most newly performed experiments. Methods 
for leveraging these public resources have focused on the interpretation of existing 
data, or analysis of new datasets independently, but do not facilitate direct comparison 
of new to existing experiments. The interpretability of common unsupervised analysis 
methods such as Principal Component Analysis would be enhanced by efficient comparison 
of the results to previously published datasets.

### Methods
To help identify replicable and interpretable axes of variation in any given gene 
expression dataset, we performed principal component analysis (PCA) on 536 studies 
comprising 44,890 RNA sequencing profiles. Sufficiently similar loading vectors, 
when compared across studies, were combined through simple averaging. We annotated 
the collection of resulting average loading vectors, which we call Replicable Axes 
of Variation (RAV), with details from the originating studies and gene set enrichment 
analysis. Functions to match PCA of new datasets to RAVs from existing studies, 
extract interpretable annotations, and provide intuitive visualization, are implemented 
as the GenomicSuperSignature R package, to be submitted to Bioconductor. 

### Results
Usecases and benchmark examples are documented in the 
[GenomicSuperSignaturePaper](https://shbrief.github.io/GenomicSuperSignaturePaper/) 
page. All the figures and tables can be reproduced using the code and 
instruction in this page as well.

### Citation
If you use GenomicSuperSignature in published research, please cite:

> Oh S, Geistlinger L, Ramos M, Blankenberg D, van den Beek M, Taroni JN, Carey VJ, Waldron L, Davis S. 
> GenomicSuperSignature facilitates interpretation of RNA-seq experiments through robust, efficient comparison to public databases. 
> *Nature Communications* **2022**;13: 3695.
> [doi: 10.1038/s41467-022-31411-3](https://www.nature.com/articles/s41467-022-31411-3)

### Other relevant code
The workflow to build the RAVmodel is available from https://github.com/shbrief/model_building which is archived in Zenodo with the identifier https://doi.org/10.5281/zenodo.6496552. All analyses presented in the GenomicSuperSignatures manuscript are reproducible using code accessible from https://github.com/shbrief/GenomicSuperSignaturePaper/ and archived in Zenodo with the identifier https://doi.org/10.5281/zenodo.6496612.

## Installation
You can install GenomicSuperSignature in Bioconductor. This can be done using 
`BiocManager`:
```
if (!require("BiocManager"))
    install.packages("BiocManager")

library(BiocManager)
install("GenomicSuperSignature")
```

RAVmodel can be directly downloaded from Google bucket with no cost. The sizes of 
RAVmodels`RAVmodel_C2.rds` and `RAVmodel_PLIERpriors.rds` are 476.1MB and 475.1MB, 
respectively. You can use `wget` or `GenomicSuperSignature::getModel` function.

```
## Download RAVmodel with wget
wget https://storage.googleapis.com/genomic_super_signature/RAVmodel_C2.rds
wget https://storage.googleapis.com/genomic_super_signature/RAVmodel_PLIERpriors.rds

## Download RAVmodel with getModel function
getModel("C2")
getModel("PLIERpriors")
```

## Schematic
#### Overview of GenomicSuperSignature 
Schematic illustration of RAVmodel construction and GenomicSuperSignature application.  Building the RAVmodel (components in grey) is performed once on a time scale of hours on a high-memory, high-storage server. Users can apply RAVmodel on their data (component in red) using the GenomicSuperSignature R/Bioconductor package (components in blue), which operates on a time scale of seconds for exploratory data analyses (components in orange) on a typical laptop computer.   

<img src="https://raw.githubusercontent.com/shbrief/GenomicSuperSignature/master/vignettes/GSig_overview.png" width="90%" height="90%"/>

<br>

#### User's perspective
The GenomicSuperSignature package allows users to access a RAVmodel (Z matrix, blue) and annotation information on each RAV. From a gene expression matrix (Y matrix, grey), users can calculate dataset-level validation score or sample score matrix (B matrix, red). Through the RAV of your interest, additional information such as related studies, GSEA, and MeSH terms can be easily extracted. 

<img src="https://raw.githubusercontent.com/shbrief/GenomicSuperSignature/master/vignettes/GSig_model_usage_diagram.png" width="90%" height="90%"/>

#### Information assembled by GenomicSuperSignature
GenomicSuperSignature connects different public databases and prior information through RAVindex, creating the knowledge graph illustrated here. Users can instantly access data and metadata resources from multiple entry points, such as gene expression profiles, MeSH terms, gene sets, and keywords. 

<img src="https://raw.githubusercontent.com/shbrief/GenomicSuperSignaturePaper/master/inst/images/GSig_knowledge_graph.png" width="90%" height="90%"/>

### 

### `studyMeta.csv`
Information on refine.bio RNAseq-compendium data downloaded on 04.10.20. You can
find out how this table is created from [here](https://github.com/shbrief/GenomicSuperSignaturePaper/blob/master/Methods/training_datasets/available_samples.Rmd).

- **studyName**: Name of the study   
- **downloaded**: The number of samples downloaded for each study   
- **metadata**: The number of samples for each study based on the metadata   
- **imported**: Whether `tximport::tximport` succeeded or not   
- **RAVmodel_677**: Whether the study is a part of 677 studies for RAVmodel (test ver.1)   
- **RAVmodel_1399**: Whether the study is a part of 1,399 studies for RAVmodel (test ver.2)   
- **RAVmodel_536**: Whether the study is a part of 536 studies for RAVmodel (production ver.),
include studies with >50 samples and successfully imported

#' PCA on gene expression profile
#'
#' Performs a principal components analysis on the given data matrix and returns
#' the results as an object of class \code{\link[stats]{prcomp}}.
#'
#' @param x a numeric or complex matrix (or data frame) which provides the gene
#' expression data for the principal components analysis. Genes in the rows and
#' samples in the columns.
#' @return A \code{\link[stats]{prcomp}} object.
#'
#' @examples
#' m = matrix(rnorm(100),ncol=5)
#' extractPC(m)
#'
#' @seealso \code{\link[stats]{prcomp}}
#' @export
extractPC <- function(x) {
    dat.pca <- stats::prcomp(t(x))
    return(dat.pca)
}

#' Subset enriched pathways of RAV
#'
#' @import methods
#'
#' @param RAVmodel PCAGenomicSignatures object. Also an output from
#' \code{\link[clusterProfiler]{GSEA}} can be used.
#' @param ind A numeric vector containing the RAV number you want to check
#' enriched pathways. If not specified, this function returns results from all
#' the RAVs.
#' @param n The number of top and bottom pathways to be selected based on
#' normalized enrichment score (NES).
#' @param both Default is \code{FALSE}, where only the top \code{n} pathways
#' will be printed. If it is set to \code{TRUE}, the output will contain both
#' top and bottom \code{n} pathways.
#' @param include_nes Defalt is \code{FALSE}. If it set to \code{TRUE}, the
#' output will include both description and NES of the enriched pathway.
#'
#' @return A DataFrame with top and bottom \code{n} pathways from the
#' enrichment results.
#'
#' @examples
#' data(miniRAVmodel)
#'
#' # all RAVS in model
#' subsetEnrichedPathways(miniRAVmodel,n=5)
#'
#' # only a specific RAV (note the colnames above)
#' subsetEnrichedPathways(miniRAVmodel,ind=695,n=5)
#'
#'
#' @export
subsetEnrichedPathways <- function(RAVmodel, ind = NULL, n = 10,
                                   both = FALSE, include_nes = FALSE) {

  ## Take the GSEA annotation
  if (is(RAVmodel, "PCAGenomicSignatures")) {
    gsea_loading <- gsea(RAVmodel)
  } else if(is.data.frame(RAVmodel) | is.matrix(RAVmodel) | is.list(RAVmodel)) {
    gsea_loading <- RAVmodel
  }

  ## RAVs to subset GSEA annotation from
  res <- list()
  if (is.null(ind)) {
    names <- names(gsea_loading)
  } else {names <- paste0("RAV", ind)}

  ## Subset the enriched pathways with top/bottom NES
  for (name in names) {
    x <- gsea_loading[[name]]
    if (is.null(dim(x))) {
      res[[name]] <- "There is no enriched pathway"
    } else {
      y <- x[,c("Description", "NES")]
      up <- y[order(y$NES, decreasing=TRUE),][seq_len(n),]
      rownames(up) <- paste0("Up_", seq_len(n))
      down <- y[order(y$NES, decreasing=FALSE),][seq_len(n),]
      rownames(down) <- paste0("Down_", seq_len(n))
      res[[name]] <- rbind(up, down)
    }
  }

  ## Remove NES if it is requested
  if (!include_nes) {
    for (name in names) {
      res[[name]] <- res[[name]][,"Description", drop = FALSE]
      colnames(res[[name]]) <- paste(name, colnames(res[[name]]), sep = ".")
    }
  }

  ## Combine list into one data frame
  res <- data.frame(res, check.names = FALSE)
  if (!both) {res <- res[seq_len(n),,drop=FALSE]}

  res <- S4Vectors::DataFrame(res)
  return(res)
}

context("Test subsetEnrichedPathways function")

data(miniRAVmodel)

test_that("Confirm the output structure", {
    res <- subsetEnrichedPathways(miniRAVmodel, ind = 695)
    val <- c("IRIS_Bcell-Memory_IgG_IgA", "DMAP_BCELLA3",
             "IRIS_Bcell-naive", "IRIS_Bcell-Memory_IgM", "DMAP_BCELLA2")

    expect_equal(dim(res), c(10, 1))
    expect_equal(colnames(res), "RAV695.Description")
    expect_equal(res[1:5,], val)

    res2 <- subsetEnrichedPathways(miniRAVmodel, ind = 695, include_nes = TRUE)
    expect_true(is.na(res2[10,1]))
})

test_that("Test `include_nes` argument", {
    res3 <- subsetEnrichedPathways(miniRAVmodel, ind = c(695, 1994),
                                   n = 3, include_nes = TRUE)
    val2 <- c("RAV695.Description", "RAV695.NES",
              "RAV1994.Description", "RAV1994.NES")

    expect_equal(colnames(res3), val2)
    expect_equal(dim(res3), c(3, 4))
    expect_equal(round(res3[,4], digits = 6), c(2.682385, 2.676483, 2.634790))
})

s2p_get_cache <- function(cache = tools::R_user_dir("GenomicSuperSignature",
                                                    "cache")) {
  BiocFileCache::BiocFileCache(cache=cache)
}

#' @importFrom BiocFileCache bfcneedsupdate bfcdownload bfcadd bfcquery bfcrpath
#'
s2p_cached_url <- function(url, rname = url, ask_on_update=FALSE,
                           ...) {
  bfc = s2p_get_cache()
  bfcres = bfcquery(bfc,rname,'rname')

  rid = bfcres$rid
  # Not found
  fileage = 0
  if(!length(rid)) {
    rid = names(bfcadd(bfc, rname, url))
  }
  # if needs update, do the download
  if(bfcneedsupdate(bfc, rid)) {
    bfcdownload(bfc, rid, ask=FALSE, ...)
    print("downloading")
  }
  bfcrpath(bfc, rids = rid)
}




#' Download a PCAGenomicSignatures model
#'
#' @param prior The name of gene sets used to annotate PCAGenomicSignatures.
#' Currently  there are two available options.
#' \itemize{
#'     \item \code{C2} : MSigDB C2 (curated gene sets)
#'     \item \code{PLIERpriors} : bloodCellMarkersIRISDMAP, svmMarkers, and
#'     canonicalPathways
#' }
#' @param load Default is \code{TRUE}. If it's set to \code{FALSE}, the models
#' are just downloaded to cache but not loaded into memory.
#' @param version Default is \code{latest}. Available versions are listed in
#' \code{version} column of \code{\link{availableRAVmodel()}} output.
#'
#' @return File cache location or PCAGenomicSignatures object loaded from it.
#'
#' @examples
#' z = getModel("C2")
#'
#' @export
getModel <- function(prior = c("C2", "PLIERpriors"), 
                     version = "latest", load = TRUE) {

    dir <- system.file("extdata", package = "GenomicSuperSignature")
    models <- read.table(file.path(dir, "availableRAVmodel.csv"), 
                         sep = ",", header=TRUE)
    models <- models[which(models$gcp == TRUE),] ## Only RAVmodels on GCP
    
    ## Check the input validity
    if (!prior %in% unique(models$prior)) {
      stop("Prior you entered isn't available.")
    }
    
    prior_ind <- which(models$prior == prior)
    if (!version %in% unique(models$version[prior_ind])) {
      stop("Version you entered doesn't exist.")
    }
    
    request_ind <- which(models$prior == prior & models$version == version)
    request_model <- paste0(models$fname[request_ind], ".rds")

    bucket_name <- "genomic_super_signature"
    fpath <- file.path("https://storage.googleapis.com",
                       bucket_name, request_model)

    fpath <- s2p_cached_url(fpath)
    if (isTRUE(load)) {
      model <- readRDS(fpath)
      return(model)
    } else {return(fpath)}
}



# Download datasets used PCAGenomicSignatures manuscript
#
# getDataset <- function() {
# }

#' Two-dimensional PCA plot with the PC annotation
#'
#' @import ggpubr
#' @import ggplot2
#'
#' @param dataset A gene expression profile to be validated. Different classes
#' of objects can be used including ExpressionSet, SummarizedExperiment,
#' RangedSummarizedExperiment, or matrix. Rownames (genes) should be in symbol
#' format. If it is a matrix, genes should be in rows and samples in columns.
#' @param RAVmodel PCAGenomicSignatures-class object
#' @param PCnum A numeric vector length of 2. The values should be 
#' between 1 and 8.
#' @param val_all The output from \code{\link{validate}}
#' @param scoreCutoff A numeric value for the minimum correlation.
#' Default 0.5.
#' @param nesCutoff A numeric value for the minimum NES. Default is \code{NULL}
#' and the suggested value is 3.
#' @param color_by A named vector with the feature you want to color by. Name
#' should be match with the sample names of the dataset.
#' @param color_lab A name for color legend. If this argument is not provided,
#' the color legend will be labeled as "Color By" by default.
#' @param trimed_pathway_len Positive inter values, which is the display width
#' of pathway names. Default is 45.
#'
#' @return Scatter plot and the table with annotation. If enriched pathway
#' didn't pass the \code{scoreCutoff} the table will be labeled as "No
#' significant pathways". If any enriched pathway didn't pass the
#' \code{nesCutoff}, it will labeled as NA.
#'
#' @examples
#' data(miniRAVmodel)
#' library(bcellViper)
#' data(bcellViper)
#' \dontrun{
#' plotAnnotatedPCA(exprs(dset), miniRAVmodel, PCnum = c(1,2))
#' }
#'
#' @export
plotAnnotatedPCA <- function(dataset, RAVmodel, PCnum, val_all = NULL,
                             scoreCutoff = 0.5, nesCutoff = NULL,
                             color_by = NULL, color_lab = NULL,
                             trimed_pathway_len = 45) {

    # Extract expression matrix from different classes
    dat <- .extractExprsMatrix(dataset)

    if (is.null(val_all)) {val_all <- validate(dat, RAVmodel)}
    PCAres <- stats::prcomp(dat)$rotation %>% as.data.frame

    # two PCs to plot
    ind1 <- which(colnames(PCAres) == paste0("PC", PCnum[1]))
    ind2 <- which(colnames(PCAres) == paste0("PC", PCnum[2]))

    if (is.null(color_lab)) {color_lab <- "Color By"}
    if (!is.null(color_by)) {
        colorFeature <- utils::stack(color_by) %>%
            tibble::column_to_rownames(var = "ind")
        PCAres <- cbind(PCAres, colorFeature)
        myPlot <- ggplot(PCAres, mapping = aes_string(x = names(PCAres)[ind1],
                                                      y = names(PCAres)[ind2],
                                                      color = "values")) +
            labs(color = color_lab) +
            geom_point()
    } else {
        myPlot <- ggplot(PCAres, mapping = aes_string(x = names(PCAres)[ind1],
                                                      y = names(PCAres)[ind2])) +
            geom_point()
    }

    # Trim the long pathway names
    annotatedPC <- annotatePC(c(ind1, ind2), val_all, RAVmodel,
                              scoreCutoff = scoreCutoff, nesCutoff = nesCutoff,
                              trimed_pathway_len = trimed_pathway_len)

    # PC annotation table - flextable version
    flextable::set_flextable_defaults(na_str = "NA")
    myTable <- flextable::flextable(annotatedPC) %>% flextable::width(width = 2.5)
    myTable <- grid::rasterGrob(flextable::as_raster(myTable))

    # # PC annotation table - ggpubr version
    # myTable <- ggpubr::ggtexttable(annotatedPC,
    #                                rows = NULL,
    #                                theme = ggpubr::ttheme("mOrange"))
    # myTable <- ggpubr::table_cell_font(myTable,
    #                                    row = 2:ggpubr::tab_nrow(myTable),
    #                                    column = 1:2, size = 9)

    res <- ggpubr::ggarrange(myPlot, myTable,
                             ncol = 1, nrow = 2, heights = c(1, 0.5))
    print(res)
}

#' Barplot GSEA output
#'
#' @param ind An interger. Index of RAV to apply GSEA.
#' @param RAVmodel PCAGenomicSignature object.
#' @param category A character vector representing MSigDB category. Options are
#' "H", "C1", "C2"(default), "C3", "C4", "C5", "C6", and "C7"
#' @param n An interger. The number of top and bottom enriched pathways to plot. Default is 10.
#' @param pvalueCutoff Cutoff for both pvalue and p.adjust. Default is 0.5.
#' @param gseaRes An output from \link{msigdb_gsea} function. If this argument
#' is provided, you don't need to provide the above inputs: ind, RAVmodel, category, n, pvalueCutoff.
#'
#' @return Barplot of GSEA output. Top and bottom \code{n} genesets based on NES
#' are plotted and qvalues are denoted by color.
#'
#' @export
gseaBarplot <- function(ind, RAVmodel, category = "C2", n = 10,
                        pvalueCutoff = 0.5, gseaRes = NULL) {

    ## Binding the variables from res locally to the function
    NES <- Description <- qvalues <- NULL

    ## GSEA results
    gseaRes <- gsea(RAVmodel)[[ind]]
    if (nrow(gseaRes) == 0) return("No pathway is enriched")   # Handle empty dataframes

    ## Barplot
    ggplot(gseaRes, aes(NES, forcats::fct_reorder(Description, NES),
                        fill = qvalues), showCategory = (n*2)) +
        geom_bar(stat = 'identity') +
        scale_fill_continuous(low = 'red', high = 'blue',
                              guide = guide_colorbar(reverse = TRUE)) +
        theme_minimal() + ylab(NULL)
}


#' Calculate jaccard similarity of two sets
#'
#' @param a A vector
#' @param b A vector
#' @return A numerical value
#'
#' @note From https://github.com/montilab/hypeR/blob/master/R/utils.R
#'
#' @keywords internal
.jaccard_similarity <- function(a, b) {
    length(intersect(a, b)) / length(union(a, b))
}

#' Calculate overlap similarity of two sets
#'
#' @param a A vector
#' @param b A vector
#' @return A numerical value
#'
#' @note From https://github.com/montilab/hypeR/blob/master/R/utils.R
#'
#' @keywords internal
.overlap_similarity <- function(a, b) {
    length(intersect(a, b)) / min(length(a), length(b))
}

#' Format a string using placeholders
#'
#' @param string A an unformatted string with placeholders
#' @param ... Variables to format placeholders with
#' @return A formatted string
#'
#' @note From https://github.com/montilab/hypeR/blob/master/R/utils.R
#'
#' @examples
#' \dontrun{
#' format_str("Format with {1} and {2}", "x", "y")
#' }
#'
#' @keywords internal
.format_str <- function(string, ...) {
    args <- list(...)
    for (i in 1:length(args)) {
        pattern <- paste("\\{", i, "}", sep="")
        replacement <- args[[i]]
        string <- gsub(pattern, replacement, string)
    }
    return(string)
}


#' Plot the network of enriched pathways
#'
#' @param ind An interger. Index of RAV to apply GSEA.
#' @param RAVmodel PCAGenomicSignature object.
#' @param category A character vector representing MSigDB category. Options are
#' "H", "C1", "C2"(default), "C3", "C4", "C5", "C6", and "C7"
#' @param n An interger. The number of top and bottom enriched pathways to plot. Default is 10.
#' @param pvalueCutoff Cutoff for both pvalue and p.adjust. Default is 0.5.
#' @param similarity_metric Metric to calculate geneset similarity. Available values
#' are \code{c("jaccard_similarity", "overlap_similarity")}.
#' @param similarity_cutoff Geneset similarity cutoff. Default is 0.3.
#' @param title Plot title
#' @return A visNetwork object
#'
#' @note Modified from \code{\link[hypeR]{hyp_emap}}
#'
#' @importFrom purrr when
#' @importFrom dplyr filter
#' @importFrom igraph graph.adjacency V
#' @importFrom visNetwork visNetwork visNodes visEdges visOptions visInteraction toVisNetworkData visIgraphLayout
#'
gseaNetwork <- function(ind, RAVmodel, category = "C2", n = 10, pvalueCutoff = 0.5,
                        similarity_metric = c("jaccard_similarity", "overlap_similarity"),
                        similarity_cutoff = 0.3, title = "") {

    ## GSEA
    gseaRes <- gsea(RAVmodel)[[ind]]
    gseaRes <- as.data.frame(gseaRes)
    if (nrow(gseaRes) == 0) return("No pathway is enriched")   # Handle empty dataframes

    # Geneset similarity matrix
    genesets <- vector(mode = "list", length = nrow(gseaRes))
    names(genesets) <- gseaRes$ID
    for (i in seq_along(genesets)) {
        genes <- stringr::str_split(gseaRes$core_enrichment[i], "/")
        genes <- unlist(genes)
        genesets[[i]] <- as.integer(genes)
    }

    genesets.mat <- sapply(genesets, function(x) {
        sapply(genesets, function(y,x) {
            if (similarity_metric == "jaccard_similarity") .jaccard_similarity(x, y)
            else if (similarity_metric == "overlap_similarity") .overlap_similarity(x, y)
            else stop(.format_str("{1} is an invalid metric", similarity_metric))
        }, x)
    })

    m <- as.matrix(genesets.mat)

    # Sparsity settings
    m[m < similarity_cutoff] <- 0

    # Similarity matrix to weighted network
    inet <- igraph::graph.adjacency(m, mode="undirected", weighted=TRUE, diag=FALSE)

    # igraph to visnet
    vnet <- visNetwork::toVisNetworkData(inet)
    nodes <- vnet$nodes
    edges <- vnet$edges

    # Add edge weights
    edges$value <- vnet$edges$weight

    # Add node scaled sizes based on genset size
    size.scaler <- function(x) (x-min(x))/(max(x)-min(x))*30
    node.sizes <- sapply(igraph::V(inet), function(x) gseaRes[x, "setSize"])
    nodes$size <-  size.scaler(node.sizes)+20

    val <- "qvalues"
    nodes$title <- sapply(igraph::V(inet), function(x) {
        paste(val, gseaRes[x, val], sep=": ")
    })

    # Add node scaled weights based on significance
    weight.scaler <- function(x) (x-max(x))/(min(x)-max(x))
    node.weights <- sapply(igraph::V(inet), function(x) gseaRes[x, val])
    nodes$color.border <- "rgb(0,0,0)"
    nodes$color.highlight <- "rgba(199,0,57,0.9)"
    nodes$color.background <- sapply(weight.scaler(node.weights), function(x) {
        if (is.na(x)) {
            return("rgba(199,0,57,0)")
        } else{
            return(paste("rgba(199,0,57,", round(x, 3), ")", sep=""))
        }
    })

    visNetwork(nodes, edges, main=list(text=title, style="font-family:Helvetica")) %>%
        visNodes(borderWidth=1, borderWidthSelected=0) %>%
        visEdges(color="rgb(88,24,69)") %>%
        visOptions(highlightNearest=TRUE) %>%
        visInteraction(multiselect=TRUE, tooltipDelay=300) %>%
        visIgraphLayout(layout="layout_nicely")
}

#' Calculate Silhouette Information of RAVs
#'
#' @description The silhouette value is a measure of how similar an object is
#' to its own cluster (cohesion) compared to other clusters (separation). The
#' silhouette width ranges from -1 to +1, where a high value indicates that
#' the object is well matched to its own cluster and poorly matched to
#' neighboring clusters.
#'
#' @param dat A matrix with all the top PCs from training data to be clustered.
#' @param kmeansRes Output from \code{stats::kmeans}.
#'
#' @return Silhouette-class object, which is an n x 3 matrix with attributes.
#' 
#' @keywords internal
#'
#' @seealso \code{\link[stats]{kmeans}}
#'
.calculateSilhouetteWidth <- function (dat, kmeansRes) {
    swRes <- cluster::silhouette(x = kmeansRes$cluster,
                                 dist = cluster::daisy(dat))
    SW <- summary(swRes)
    return(SW)
}



#' Calculate average loadings of each cluster
#'
#' @param dat A data frame. Each row represents principle components from
#' different training datasets. Columns are genes used for PCA analysis.
#' @param k The number of clusters used for hierarchical clustering
#' @param n The number of top principle components from each datasets used for
#' model building. Default is 20.
#' @param study Under default (\code{TRUE}), studies involved in each cluster
#' will be added in the output.
#' @param cluster Provide pre-defined cluster membership of your data.
#'
#' @return
#' A named list of 6 elements is returned. It contains:
#' \describe{
#'    \item{\code{cluster}}{A numeric vector on cluster membership of PCs}
#'    \item{\code{size}}{A integer vector on the size of clusters}
#'    \item{\code{avgLoading}}{A matrix of average loadings. Columns for
#'    clusters and rows for genes}
#'    \item{\code{k}}{The number of clusters}
#'    \item{\code{n}}{The number of top PCs used for clustering}
#'    \item{\code{studies}}{A list of character vector containing studies in
#'    each cluster}
#' }
#'
#' @examples
#' data(miniAllZ)
#' data(res_hcut)
#' res <- buildAvgLoading(miniAllZ, k = 40, cluster = res_hcut$cluster)
#'
#' @export
buildAvgLoading <- function(dat, k, n = 20, cluster = NULL, study = TRUE) {

    # Input validation
    if (missing(k)) {stop("`k` must be provided.")}
    if (!is.null(cluster)) {
        k <- length(unique(cluster))
        x <- table(cluster) %>% as.data.frame()
        res <- list(cluster = cluster, size = x$Freq)
    } else {
        stop("Cluster membership of elements should be provided 
             through 'cluster' argument.")
    }
    stopifnot(length(study) == 1L, !is.na(study), is.logical(study))


    # Separate the PC table into each cluster
    cl_ls <- vector(mode = "list", length = k)
    for (i in seq_len(k)) {
        datName <- paste0("Cl", k, "_",
                          formatC(i, width = 2, format = "d", flag = "0"))
        cl_ls[[i]] <- dat[, res$cluster==i, drop=FALSE] %>% t
        names(cl_ls)[i] <- datName
    }

    # the number of unique datasets in each cluster
    unique_sets <- vector(length = k)
    for (i in seq_len(k)) {
        dat <- cl_ls[[i]]
        dataSetName <- gsub(".PC\\d+$", "", rownames(dat))
        uniqueDataSetName <- length(unique(dataSetName))
        unique_sets[i] <- uniqueDataSetName
    }

    # Calculate the average of loadings in each cluster
    names(cl_ls) <- paste0(names(cl_ls), " (", res$size, "/", unique_sets, ")")
    l <- ncol(dat)   # the number of genes for average loading
    avg.loadings <- vapply(cl_ls, colMeans, FUN.VALUE = numeric(l))

    # Save avgloading, and 'studies in cluster'
    res$avgLoading <- as.matrix(avg.loadings)
    res$k <- k
    res$n <- n

    # # Silhouette Width
    # sw <- .calculateSilhouetteWidth(dat, res)
    # res$sw <- sw

    if (study) {res$studies <- findStudiesInCluster(res)}
    return(res)
}

#' Select enriched pathways for RAVmodel
#'
#' From the GSEA output of each RAV, subset the enriched pathways with the
#' minimum q-value. Also, this function keeps only \code{Description, NES, qvalues}.
#'
#' @param RAVmodel PCAGenomicSignatures object.
#' @param gsea.dir "~/data2/PCAGenomicSignatureLibrary/refinebioRseq/RAVmodel_536/gsea"
#' @return A list with the length of RAVs in the provided RAVmodel. Each
#' element contains the subset data.frame of GSEA output.
#'
#' @note This function is for model construction, not for end-users.
#'
searchPathways <- function(RAVmodel, gsea.dir) {

  ## If you want to select only a subset of RAVs with the specific cluster size
  # ind <- which(metadata(RAVmodel)$size > 3)
  # gsea_all <- vector(mode = "list", length = length(ind))
  # names(gsea_all) <- colnames(RAVmodel)[ind]

  gsea_all <- vector(mode = "list", length = ncol(RAVmodel))
  names(gsea_all) <- colnames(RAVmodel)
  gsea.dir <- gsea.dir

  for (i in seq_len(ncol(RAVmodel))) {
    pathToRes <- file.path(gsea.dir, paste0("gsea_", i, ".rds"))
    res <- readRDS(pathToRes)

    # If there is no enriched pathways
    if (nrow(res) == 0) {
      resName <- paste0("RAV", i)
      gsea_all[[resName]] <- res[, c("Description", "NES", "pvalue", "qvalues"), drop = FALSE]
      print(paste("RAV", i, "has no enriched pathways."))
      next
    }

    res <- res[which(res$qvalues == min(res$qvalues)), c("Description", "NES", "pvalue", "qvalues"), drop = FALSE]
    resName <- paste0("RAV", i)
    gsea_all[[resName]] <- res
    print(paste("RAV", i, "is added."))
  }

  return(gsea_all)
}

### ==============================================
### GenomicSignatures-Class
### ==============================================
#' GenomicSignatures
#' @name GenomicSignatures-class
#' @title Virtual class inherited from SummarizedExperiment
#'
#' @description GenomicSignatures is a virtual class inherited from
#' SummarizedExperiment and hosts GenomicSignatures models built from different
#' dimensional reduction methods. Currently, PCA-based model, called
#' PCAGenomicSignatures, is available.
#'
#' @param x A \code{GenomicSignatures-class} object
#' @param value See details.
#'
#' @import SummarizedExperiment
#' @docType class
#' @exportClass GenomicSignatures
setClass("GenomicSignatures",
         contains = c("SummarizedExperiment", "VIRTUAL")
)

### ==============================================
### PCAGenomicSignatures-Class
### ==============================================
#' PCAGenomicSignatures
#' @name PCAGenomicSignatures-class
#' @title PCAGenomicSignatures-class
#'
#' @description PCA-based \code{\link{GenomicSignatures-class}}.
#'
#' @slot trainingData A \code{\link[S4Vectors]{DataFrame}} class object for
#' metadata associated with training data
#' @param x A \code{GenomicSignatures-class} object
#' @param value See details.
#'
#' @docType class
#'
#' @examples
#' data(miniRAVmodel)
#' miniRAVmodel
#'
#' @exportClass PCAGenomicSignatures
.PCAGenomicSignatures <- setClass("PCAGenomicSignatures",
         slots = list(
             trainingData = "DataFrame" 
         ),
         contains = "GenomicSignatures"
)

#' @include GenomicSignatures-class.R

#' @name GenomicSignatures-methods
#' @title Methods and accesors for \code{GenomicSignatures} object
#'
#' @description The default contents of \code{GenomicSignatures} object, with
#' a set of getter and setter generic functions, which extract either the
#' \code{assay}, \code{colData}, or \code{metadata} slots of a
#' \code{\link{GenomicSignatures-class}} object. When you create this object,
#' \code{colData$studies} should be populated before adding any information in
#' \code{trainingData} slot.
#'
#' @details
#' \itemize{
#'     \item assay(x) : RAVindex (= avgLoadings) containing genes x RAVs
#'     \item metadata(x) : Metadata associated with RAVindex building process
#'     \item colData(x) : Information on RAVs
#' }
#'
#' @section Setters:
#' Setter method values (i.e., \code{function(x) <- value}):
#' \itemize{
#'     \item metadata<- : Assign metadata
#'     \item coldata<- : Assign extra information associated with RAVs
#'     \item geneSets<- : A character vector containing the name of gene sets
#'     used to annotate average loadings
#'     \item updateNote<- : A character vector. Describes the main feature of a
#'     model construction
#' }
#'
#' @section Getters:
#' \itemize{
#'    \item RAVindex : Equivalent to \code{assays(x)$RAVindex}
#'    \item geneSets : Access the \code{metadata(x)$geneSets} slot
#'    \item updateNote : Access the \code{metadata(x)$updateNote} slot
#'    \item version : Access the \code{metadata(x)$version} slot
#' }
#'
#' @param x A \code{GenomicSignatures} object
#' @param value See details.
#'
#' @return A GenomicSignatures object for the constructor
#'
#' @examples
#' data(miniRAVmodel)
#' miniRAVmodel
#'
#'
#' @aliases RAVindex colData metadata geneSets updateNote
#' geneSets<- updateNote<-
NULL


### ----------------------------------------------
### Getter
### ----------------------------------------------

#' @export
setGeneric("RAVindex", function(x) standardGeneric("RAVindex"))

#' @exportMethod RAVindex
#' @rdname GenomicSignatures-methods
setMethod("RAVindex", "GenomicSignatures", function(x) {
    out <- assay(x)
    return(out)
})

#' @export
setGeneric("geneSets", function(x) standardGeneric("geneSets"))

#' @exportMethod geneSets
#' @rdname GenomicSignatures-methods
setMethod("geneSets", "GenomicSignatures", function(x) {
    out <- metadata(x)$geneSets
    return(out)
})

#' @export
setGeneric("updateNote", function(x) standardGeneric("updateNote"))

#' @exportMethod updateNote
#' @rdname GenomicSignatures-methods
setMethod("updateNote", "GenomicSignatures", function(x) {
    out <- metadata(x)$updateNote
    return(out)
})

#' @export
setGeneric("version", function(x) standardGeneric("version"))

#' @exportMethod version
#' @rdname GenomicSignatures-methods
setMethod("version", "GenomicSignatures", function(x) {
    out <- metadata(x)$version
    return(out)
})



### ----------------------------------------------
### Setter
### ----------------------------------------------

#' @export
setGeneric("geneSets<-", function(x, value) standardGeneric("geneSets<-"))

#' @exportMethod geneSets<-
#' @rdname GenomicSignatures-methods
setMethod("geneSets<-", "GenomicSignatures", function(x, value) {
    metadata(x)$geneSets <- value
    return(x)
})

#' @export
setGeneric("updateNote<-", function(x, value) standardGeneric("updateNote<-"))

#' @exportMethod updateNote<-
#' @rdname GenomicSignatures-methods
setMethod("updateNote<-", "GenomicSignatures", function(x, value) {
    metadata(x)$updateNote <- value
    return(x)
})

#' Annotate top PCs from the dataset
#'
#' This function finds the RAV with the highest validation score (including
#' RAVs with negative silhouette width) for specified PC of the dataset and
#' returns the top enriched pathways.
#'
#' @param PCnum A numeric vector. PC number of your dataset to retrieve
#' annotation results for. The vector can contain any integer number among
#' \code{1:8}.
#' @param val_all The output from \code{\link{validate}}
#' @param RAVmodel The RAVmodel used to generate the input for the argument,
#' \code{val_all}.
#' @param n An integer. Default is 5. The number of the top enriched pathways
#' to print out. If there are fewer than n pathways passed the cutoff, it will
#' print out \code{NA}.
#' @param scoreCutoff A numeric value for the minimum correlation between loadings
#' of the dataset principal component and the RAV. Default is 0.5.
#' @param nesCutoff A numeric value for the minimum Normalized Enrichment Score
#' (NES) for the enrichment analysis. Default is \code{NULL}
#' The suggested value is 3.
#' @param simplify A logical. Under default (\code{TRUE}), the output will be a
#' data frame with the number of column same as the length of \code{PCnum}
#' argument, and the number of row same as the \code{n} argument. If it is set
#' to \code{FALSE}, the output will be a list with the length of \code{PCnum}
#' argument, where each element is a data frame containing detailed GSEA output
#' of enriched pathways.
#' @param abs Default is \code{FALSE}. If it's set to \code{TRUE}, the enriched
#' pathways will be listed based on absolute value of the Normalized Enrichment
#' Score (NES).
#' @param trimed_pathway_len Positive integer values, which is the display width
#' of pathway names. Default is 45.
#'
#' @return A data frame of a list based on the \code{simplify} argument. Check
#' the output detail above.
#'
#' @examples
#' data(miniRAVmodel)
#' library(bcellViper)
#' data(bcellViper)
#' val_all <- validate(dset, miniRAVmodel)
#' annotatePC(2, val_all, miniRAVmodel)
#'
#' @export
annotatePC <- function(PCnum, val_all, RAVmodel, n = 5,
                       scoreCutoff = 0.5, nesCutoff = NULL,
                       simplify = TRUE, abs = FALSE,
                       trimed_pathway_len = 45) {

  ## Input validation
  if(any(!PCnum %in% seq(8))) {stop("PCnum should be an integer among c(1:8).")}
  stopifnot(length(simplify) == 1L, !is.na(simplify), is.logical(simplify))
  stopifnot(length(abs) == 1L, !is.na(abs), is.logical(abs))

  res <- vector(mode = "list", length = length(PCnum))

  for (i in seq_along(PCnum)) {
    annotPC <- validatedSignatures(val_all, num.out = 1, RAVmodel,
                                   scoreCutoff = scoreCutoff,
                                   whichPC = PCnum[i])
    cl_name <- paste0("RAV", annotPC[,"cl_num"])
    annotatedCluster <- gsea(RAVmodel)[[cl_name]]

    if (is.null(annotatedCluster)) {
      emptyTable <- gsea(RAVmodel)[[1]][0,] # assign the empty GSEA table
      emptyTable[seq(n),1] <- "No significant pathways"
      res[[i]] <- emptyTable
      names(res)[i] <- paste0("PC", PCnum[i], "-noAnnot")
    } else {
      # absolute value of NES
      FUN <- function(x) {if (abs) {abs(x)} else {I(x)}}

      # apply NES cutoff
      if (!is.null(nesCutoff)) {
        annotatedCluster <- annotatedCluster[
          FUN(annotatedCluster$NES) >= nesCutoff,,drop = FALSE]
      }

      # subset GSEA results
      topAnnotation <- annotatedCluster[
        order(FUN(annotatedCluster$NES), decreasing = TRUE),,drop = FALSE]
      topAnnotation <- topAnnotation[seq_len(n),]
      rownames(topAnnotation) <- NULL

      res[[i]] <- topAnnotation
      names(res)[i] <- paste0("PC", PCnum[i], "-", cl_name)
    }
  }

  # Trim the long pathway names
  for (i in seq_along(res)) {
    ind <- which(nchar(res[[i]]$Description) > trimed_pathway_len)
    res[[i]]$Description[ind] <- paste0(strtrim(res[[i]]$Description[ind],
                                                trimed_pathway_len), "...")
  }

  # Output formats:
  # A list of data frames with details vs. data frame with description only
  if (simplify) {
    simple_res <- lapply(res, function(x) x$Description) %>% as.data.frame
    return(simple_res)
  } else {
    return(res)
  }
}

context("Test annotatePC function")

data(miniRAVmodel)
library(bcellViper)
data(bcellViper)

val_all <- validate(dset, miniRAVmodel)

test_that("Users should choose one of the top 8 PCs of their dataset", {
  expect_message(res <- annotatePC(2, val_all, miniRAVmodel))
  expect_equal(names(res), "PC2.RAV1076")

  res2 <- annotatePC(2, val_all, miniRAVmodel, trimed_pathway_len = 10)
  expect_equal(res2[2,], "REACTOME_H...")
  expect_error(annotatePC(9, val_all, miniRAVmodel),
               "PCnum should be an integer among c\\(1:8\\).")
  expect_error(annotatePC(c(1:2, 9), val_all, miniRAVmodel),
               "PCnum should be an integer among c\\(1:8\\).")
})

test_that("Extract multiple PCs", {
  res3 <- annotatePC(2:4, val_all, miniRAVmodel, simplify = FALSE)
  expect_true(is.list(res3))
  expect_equal(names(res3), c("PC2-RAV1076", "PC3-noAnnot", "PC4-noAnnot"))

  res4 <- annotatePC(2:4, val_all, miniRAVmodel, n = 7)
  expect_true(is.data.frame(res4))
  expect_equal(dim(res4), c(7,3))
})

test_that("Using absolute NES for subsetting", {
  res5 <- annotatePC(2:4, val_all, miniRAVmodel, nesCutoff = 1, abs = TRUE)
  expect_equal(res5[1,1], "REACTOME_CELL_CYCLE")

  res6 <- annotatePC(2, val_all, miniRAVmodel, nesCutoff = 1, abs = FALSE)
  expect_true(is.na(res6[1,]))

  res7 <- annotatePC(2, val_all, miniRAVmodel, abs = FALSE, simplify = FALSE)
  expect_equal(res7[[1]][1,1], "REACTOME_METABOLISM_OF_PROTEINS")
})

suppressPackageStartupMessages(library(optparse))

### Parsing command line -------------------------------------------------------
option_list <- list(
    make_option(c("--input"), type = "character",
                default = NULL, help = "Count matrix in tsv format"),
    make_option(c("--model"), type = "character",
                default = NULL, help = "RAVmodel to apply.
                Currently 'C2' and 'PLIERpriors' are available"),
    make_option(c("--method"), type = "character",
                default = formals(GenomicSuperSignature::validate)$method),
    make_option(c("--maxFrom"), type = "character",
                default = formals(GenomicSuperSignature::validate)$maxFrom),
    make_option(c("--level"), type = "character",
                default = formals(GenomicSuperSignature::validate)$level),
    make_option(c("--scale"), type = "character",
                default = formals(GenomicSuperSignature::validate)$scale),
    make_option(c("--outDir"), type = "character",
                default = NULL, help = "Output file name"),
    make_option(c("--validate"), type = "character",
                default = NULL, help = "Path to save validate.csv"),
    make_option(c("--sampleScore"), type = "character",
                default = NULL, help = "Path to save sampleScore.csv"),
    make_option(c("--html"), type = "character",
                default = NULL, help = "Path to save HTML report"),
    make_option(c("--numOut"), type = "integer",
                default = 3, help = "The number of top validated RAVs to check"),
    make_option(c("--toolDir"), type = "character",
                default = ".", help = "Directory containing the tool scripts (e.g. gss.Rmd")
)

opt <- parse_args(OptionParser(option_list = option_list),
                  args = commandArgs(trailingOnly = TRUE))
input <- opt$input
model <- opt$model
out_dir <- opt$outDir
num_out <- opt$numOut

if (is.null(input)) stop("Need --input.")
if (is.null(model)) stop("Need --model.")
if (is.null(out_dir)) stop("Need --outDir.")

input_name <- basename(tools::file_path_sans_ext(input))
out_dir <- normalizePath(out_dir)

suppressPackageStartupMessages(library(GenomicSuperSignature))
dat <- as.matrix(read.table(file = input, header = TRUE, sep = "\t",
                            row.names = 1))
if (model %in% c("C2", "PLIERpriors")) {
    rav_model <- getModel(model)
} else {
    rav_model <- readRDS(model)
}



### validate -------------------------------------------------------------------
val_all <- validate(dat, rav_model)
validated_ind <- validatedSignatures(val_all, num.out = num_out,
                                     swCutoff = 0, indexOnly = TRUE)
n <- min(num_out, length(validated_ind), na.rm = TRUE)

### sample score ---------------------------------------------------------------
score <- calculateScore(dat, rav_model)

### Save tables in csv ---------------------------------------------------------
# Validation
if (is.null(opt$validate)) {
    output_fname <- file.path(out_dir, paste0(input_name, "_validate.csv"))
} else {
    output_fname <- opt$validate
}
write.csv(val_all,
          file = output_fname,
          row.names = TRUE)

# Sample Score
if (is.null(opt$sampleScore)) {
    output_fname <- file.path(out_dir, paste0(input_name, "_sampleScore.csv"))
} else {
    output_fname <- opt$sampleScore
}
write.csv(score,
          file = output_fname,
          row.names = TRUE)

# GSEA
for (i in seq_len(n)) {
    rav_num <- validated_ind[i]
    rav_name <- paste0("RAV", rav_num)
    res <- gsea(rav_model)[[rav_name]]

    output_fname <- paste0(input_name, "_genesets_RAV", rav_num, ".csv")
    write.csv(res,
              file = file.path(out_dir, output_fname),
              row.names = TRUE)
}

# Related prior studies
for (i in seq_len(n)) {
    rav_num <- validated_ind[i]
    res <- findStudiesInCluster(rav_model, rav_num, studyTitle = TRUE)

    output_fname <- paste0(input_name, "_literatures_RAV", rav_num, ".csv")
    write.csv(res,
              file = file.path(out_dir, output_fname),
              row.names = TRUE)
}

### Create a report ------------------------------------------------------------
if (is.null(opt$html)) {
    output_fname <- file.path(out_dir, paste0("GSS-", input_name, "-",
                              format(Sys.Date(), format = "%Y%m%d"), ".html"))
} else {
    output_fname <- opt$html

}
rmarkdown::render(
    file.path(opt$toolDir, "gss.Rmd"), params = list(
        val_all = val_all,
        dat = dat,
        RAVmodel = rav_model,
        inputName = input_name,
        numOut = num_out,
        score = score
    ),
    output_file = output_fname,
    intermediates_dir = ".",
    knit_root_dir = "."
)

### Extract expression matrix from different classes of input datasets
.extractExprsMatrix <- function(dataset) {
  if (is(dataset, "ExpressionSet")) {
    dat <- Biobase::exprs(dataset)
  } else if (is(dataset,"SummarizedExperiment")) {
    dat <- SummarizedExperiment::assay(dataset)
  } else if (is.matrix(dataset)) {
    dat <- dataset
  } else {
    stop("'dataset' should be one of the following objects: ExpressionSet,
         SummarizedExperiment, and matrix.")
  }
  return(dat)
}


### Check ind validity
.availableRAV <- function(RAVmodel, ind) {
  availableRAV <- gsub("RAV", "", colData(RAVmodel)$RAV) %>% as.numeric

  ## Check whether the ind exists in the model
  x <- vector(length = length(ind))
  for (i in seq_along(ind)) {
    if (!ind[i] %in% availableRAV) {
      x[i] <- TRUE   # assign TRUE if index doesn't exist
    }
  }

  ## Print error message if any of the ind doesn't exist.
  if (any(x)) {
    y <- paste(paste0("RAV",ind[x]), collapse=", ")  # combine non-existing ind
    msg <- paste0("Selected ind (", y, ") doesn't exist.")
    stop(msg)
  }
}


## Restructure RAVmodel metadata slot
.RAVmodelVersion <- function(RAVmodel) {
  if (version(RAVmodel) == ">= 0.0.7") {
    cluster <- S4Vectors::metadata(RAVmodel)$cluster
  } else {
    cluster <- colData(RAVmodel)$cluster
  }
}


## Extract variance explained by PCs in a given cluster
.varByPCsInCluster <- function(RAVmodel, ind) {
  # components in clusters
  cl_membership <- metadata(RAVmodel)$cluster
  components <- names(which(cl_membership == ind))

  # PCA summary
  pcaSummary <- trainingData(RAVmodel)$PCAsummary
  Projs <- lapply(components, function(x) {
    unlist(strsplit(x, "\\.PC"))[1] %>% as.character
  }) %>% unlist
  data <- pcaSummary[Projs]

  # Extract variance explained
  input_summary <- as.data.frame(matrix(ncol = 3, nrow = length(data)))
  colnames(input_summary) <- c("studyName", "PC", "Variance explained (%)")

  for (i in seq_along(data)) {
    studyname <- Projs[i]
    j <- unlist(strsplit(components[i], "\\.PC"))[2] %>% as.numeric
    var <- data[[i]]["Variance",j]

    input_summary[i, 1] <- studyname
    input_summary[i, 2] <- j
    input_summary[i, 3] <- round(var*100, digits = 2)
  }

  return(input_summary)
}


## Message for low-quality RAVs
.lowQualityRAVs <- function(RAVmodel, ind, filterMessage = TRUE) {

  if (isTRUE(filterMessage)) {
    ## Load filterList
    local_data_store <- new.env(parent = emptyenv())
    data("filterList", envir = local_data_store, package = "GenomicSuperSignature")
    filterList <- local_data_store[["filterList"]]

    ## Select RAVmodel
    filterListNames <- c("Cluster_Size_filter", "GSEA_C2_filter",
                         "GSEA_PLIERpriors_filter", "Redundancy_filter")
    c2 <- "MSigDB C2 version 7.1"
    plier_priors <- "Three priors from PLIER (bloodCellMarkersIRISDMAP, svmMarkers, and canonicalPathways)"

    if (nrow(trainingData(RAVmodel)) == 536 & geneSets(RAVmodel) == c2) {
      filterList <- filterList[filterListNames[c(1,2,4)]]
    } else if ((nrow(trainingData(RAVmodel)) == 536 &
                geneSets(RAVmodel) == plier_priors)) {
      filterList <- filterList[filterListNames[c(1,3,4)]]
    }

    ## Check whether index belong to the filter list
    for (i in ind) {
      res <- vapply(filterList, function(x) {i %in% x}, logical(1))
      if (any(res)) {
        filtered <- paste(names(res)[which(res == TRUE)], collapse = ", ") %>%
          gsub("_filter", "", .)
        msg <- paste(paste0("RAV", i), "can be filtered based on", filtered)
        message(msg)
      }
    }

    ## More information on GenomicSuperSignaturePaper GitHub page
    # if (any(res)) {message("Information on filtering : bit.ly/rav_filtering")}
  }
}


## Study metadata for different RAVmodels
.getStudyMeta <- function(RAVmodel) {

  td <- rownames(trainingData(RAVmodel)) # training data used for RAVmodel
  dir <- system.file("extdata", package = "GenomicSuperSignature")

  if ("DRP000987" %in% td) {
    ## 536 datasets from refine.bio
    studyMeta <- utils::read.table(file.path(dir, "studyMeta.tsv.gz"))
  } else if ("GSE13294" %in% td) {
    ## 8 CRC and 10 OV from curated data packages
    studyMeta <- utils::read.table(file.path(dir, "studyMeta_CRCOV.tsv"))
  }

  return(studyMeta)
}

#' Remove rows with missing and Inf values from a matrix
#'
#' @param x A numeric matrix.
#' @return The updated input matrix where rows with
#'    NA and Inf values are removed.
#'
#' @examples
#' m = matrix(rnorm(100),ncol=10)
#' m[1,1] = NA
#'
#' m1 = rmNaInf(m)
#' dim(m1)
#'
#' @export
rmNaInf <- function(x) {
  if(!is.matrix(x)) stop("x must be a matrix (expression values)!")
  x <- x[apply(x, 1, function(row) {
    !any(is.na(row) | row %in% c(Inf, -Inf))}), ]
  return(x)
}

.filterBy <- function(data, cutoff, type = c("cl_size", "score", "sw")) {
  type <- match.arg(type)
  ind <- rownames(data) == type
  val_ind <- which(data[ind,] >= cutoff)
  data[,val_ind]
}

.validatedSignaturesForMulipleStudies <- function(data, num.out = num.out,
                                                  scoreCutoff = NULL) {
  studies <- rownames(data)
  ind <- grep("_PC", studies)
  data <- data[-ind,]

  if (!is.null(scoreCutoff)) {scoreCutoff <- scoreCutoff}
  else {scoreCutoff <- 0.7}   # default cutoff for multiple-studies case

  above_cutoff <- apply(data, 2, function(x) {any(x > scoreCutoff)})
  data <- data[, above_cutoff, drop=FALSE]
  return(data)
}


#' Validation result in data frame
#'
#' @param val_all An output matrix from \code{\link{validate}} function. If this
#' input is from multiple datasets, only \code{scoreCutoff} argument will be
#' considered and other inputs will be ignored.
#' @param RAVmodel PCAGenomicSignatures-class object. RAVmodel used to prepare
#' \code{val_all} input.
#' @param num.out A number of highly validated RAVs to output. Default is 5.
#' If any of the cutoff parameters are provided, \code{num.out} or the number of
#' filtered RAVs, whichever smaller, will be chosen.
#' @param scoreCutoff A numeric value for the minimum correlation. For
#' multi-studies case, the default is 0.7.
#' @param swCutoff A numeric value for the minimum average silhouette width.
#' @param clsizeCutoff An integer value for the minimum cluster size.
#' @param indexOnly A logical. Under the default (= \code{FALSE}), the detailed
#' information on validated RAVs, such as score, average silhouette width,
#' cluster size, is printed. If it is set TRUE, only the RAV number will be
#' printed.
#' @param whichPC An integer value between 1 and 8. PC number of your data to
#' check the validated signatures with. Under the default (\code{NULL}), it
#' outputs top scored signatures with any PC of your data.
#' @param filterMessage A logical. Under the default \code{TRUE}, any output
#' RAV belong to the filtering list will give a message. Silence this message
#' with \code{filterMessage=FALSE}. You can check the filter list using
#' \code{data("filterList")}.
#'
#' @return A subset of the input matrix, which meets the given condition.
#'
#' @examples
#' data(miniRAVmodel)
#' library(bcellViper)
#' data(bcellViper)
#' val_all <- validate(dset, miniRAVmodel)
#' validatedSignatures(val_all, miniRAVmodel, num.out = 3, scoreCutoff = 0)
#'
#' @export
validatedSignatures <- function(val_all, RAVmodel,
                                num.out = 5, scoreCutoff = NULL,
                                swCutoff = NULL, clsizeCutoff = NULL,
                                indexOnly = FALSE, whichPC = NULL,
                                filterMessage = TRUE) {
  # Input validation
  stopifnot(length(indexOnly) == 1L, !is.na(indexOnly), is.logical(indexOnly))

  data <- t(val_all)
  ind <- which(rownames(data) == "score")
  pc_ind <- which(rownames(data) == "PC")

  # If the validation result is from the list of datasets
  if (length(ind) == 0) {
    res <- .validatedSignaturesForMulipleStudies(data, scoreCutoff=scoreCutoff)

    if (!indexOnly) {
      return(res)
    } else {
      res_ind <- gsub("RAV", "", colnames(res))
      res_ind <- as.numeric(res_ind)
      return(res_ind)
    }
    stop
  }

  if (!is.null(whichPC)) {
    if (!whichPC %in% seq_len(8)) {
      stop("whichPC should be an integer between 1 and 8.")
    }
    subset_ind <- which(data[pc_ind,] == whichPC)
    data <- data[,subset_ind, drop = FALSE]
  }

  if (!is.null(scoreCutoff)) {
    score_subset <- .filterBy(data, scoreCutoff, "score")
  } else {score_subset <- data}
  if (!is.null(swCutoff)) {
    sw_subset <- .filterBy(data, swCutoff, "sw")
  } else {sw_subset <- data}
  if (!is.null(clsizeCutoff)) {
    clsize_subset <- .filterBy(data, clsizeCutoff, "cl_size")
  } else {clsize_subset <- data}

  common_col <- intersect(colnames(score_subset), colnames(sw_subset))
  common_col <- intersect(common_col, colnames(clsize_subset))

  if (length(common_col) == 0) {
    dat <- data[, 0]
  } else {
    common_subset <- data[, common_col]
    ordered_ind <- order(common_subset[ind,], decreasing = TRUE)
    n_out <- min(num.out, ncol(common_subset))
    ordered_ind <- ordered_ind[seq_len(n_out)]
    dat <- common_subset[,ordered_ind, drop = FALSE]
  }

  ## Check the RAV quality
  dat_ind <- dat["cl_num",]
  .lowQualityRAVs(RAVmodel, dat_ind, filterMessage)

  ## Output as a data frame vs. a vector (of index only)
  if (!indexOnly) {
    return(t(dat))
  } else {
    validatedIndex <- dat["cl_num",]
    validatedIndex <- as.numeric(validatedIndex)
    return(validatedIndex)
  }
}

data(miniRAVmodel)
library(bcellViper)
data(bcellViper)
val_all <- validate(dset, miniRAVmodel)

test_that("validatedSignatures.R", {
    res <- validatedSignatures(val_all, miniRAVmodel, 
                               num.out = 3, scoreCutoff = 0)
    res2 <- validatedSignatures(val_all, miniRAVmodel,
                                num.out = 3, scoreCutoff = 0, indexOnly = TRUE)
    res3 <- validatedSignatures(val_all, miniRAVmodel,
                                num.out = 10, scoreCutoff = 0.575)
    
    expect_true(is.matrix(res))
    expect_true(is.vector(res2))
    expect_true(is.numeric(res2))
    expect_equal(res2, c(1076, 2538, 338))
    expect_equal(dim(res3), c(2, 5))
})

#' Find the RAVs with the keyword-containing enriched pathways
#'
#' This function finds RAVs containing the keyword you provide. If you provide
#' "the number of keyword-containing pathways per RAV" in argument \code{k},
#' it will give you the RAV number.
#'
#' @param RAVmodel PCAGenomicSignatures-object
#' @param keyword A character vector. If you are searching for multiple keywords
#' at the same time, use \code{\link{paste}} with \code{collapse="|"} argument.
#' @param n The number of top ranked (based on abs(NES)) pathways you want to
#' search your keyword
#' @param k The number of keyword-containing pathways you want to get the RAV
#' number. Under default (\code{NULL}), the output will be a data frame with two
#' columns: '# of keyword-containing pathways' and 'Freq'. If you assign the
#' value for this argument, the output will be an integer vector containing the
#' RAV index.
#'
#' @return A data frame or integer vector depending on the parameter \code{k}.
#'
#' @examples
#' data(miniRAVmodel)
#' findSignature(miniRAVmodel, "Bcell")
#' findSignature(miniRAVmodel, "Bcell", k = 5)
#'
#' @export
findSignature <- function(RAVmodel, keyword, n = 5, k = NULL) {
  gsea_all <- gsea(RAVmodel)

  # top n pathways based on abs(NES) value
  topPathways <- sapply(gsea_all, function(x) {
    y <- x[order(abs(x$NES), decreasing = TRUE),][seq_len(n),]
    grep(keyword, y[["Description"]], ignore.case = TRUE)
  })

  nTopPathways <- vapply(topPathways, length, FUN.VALUE = integer(1))
  res <- nTopPathways %>% table %>% as.data.frame
  colnames(res)[1] <- c("# of keyword-containing pathways")

  if (is.null(k)) {
    return(res)
  } else {
    if (!k %in% res[,1]) {
      msg <- paste("There is no RAV with", k,
                   "keyword-containing, enriched pathways.")
      warning(msg)
    } else {
      res <- which(nTopPathways == k) %>% as.numeric
      return(res)
    }
  }
}



#' Find the rank of your keyword in the RAV's GSEA annotation
#'
#' Once you provide RAVmodel, keyword you're searching for, and the RAV number
#' to this function, it will give you the abs(NES)-based rank of your keyword in
#' the enriched pathways of the target RAV. If can be useful to find out how
#' uniquely your keyword-containing pathways are represented.
#'
#' @param RAVmodel PCAGenomicSignatures-object.
#' @param keyword A character vector. If you are searching for multiple keywords
#' at the same time, use \code{\link{paste}} with \code{collapse="|"} argument.
#' @param ind An integer. The RAV number you want to check.
#' @param n An integer. The number of top enriched pathways (based on abs(NES))
#' to search. Under default (\code{NULL}), all the enriched pathways are used.
#' @param includeTotal Under the default condition (\code{TRUE}), the total
#' number of enriched pathways will be also printed out as a part of the output.
#'
#' @return A character containing the rank of keyword-containing pathways
#' (separated by |), followed by the total number of enriched pathways in
#' parenthesis.
#'
#' @examples
#' data(miniRAVmodel)
#' findKeywordInRAV(miniRAVmodel, "Bcell", ind = 695)
#'
#' @export
findKeywordInRAV <- function(RAVmodel, keyword, ind,
                             n = NULL, includeTotal = TRUE) {
 
   .availableRAV(RAVmodel, ind)
  name <- paste0("RAV", ind)
  gsea <- gsea(RAVmodel)[[name]]
  total <- nrow(gsea)

  gsea <- gsea[order(abs(gsea$NES), decreasing = TRUE),,drop = FALSE]
  if (!is.null(n)) {gsea <- gsea[seq_len(n),]}

  keywordRank <- grep(keyword, gsea$Description, ignore.case = TRUE)
  if (length(keywordRank) == 0) {
    msg <- paste(name, "doesn't have any pathway with the keyword,", keyword)
    warning(msg)
  } else {
    if (!includeTotal) {
      res <- paste(keywordRank, collapse = "|")
    } else {
      res <- paste0(paste(keywordRank, collapse = "|"), " (out of ", total, ")")
    }
    return(res)
  }
}

context("Test functions in findSignature.R file")

data(miniRAVmodel)
library(bcellViper)
data(bcellViper)
val_all <- validate(dset, miniRAVmodel)

test_that("findSignature function", {
    res <- findSignature(miniRAVmodel, keyword = "Bcell")
    res2 <- findSignature(miniRAVmodel, keyword = "Bcell", k = 5)

    expect_true(is.data.frame(res))
    expect_true(is.vector(res2))
    expect_true(is.numeric(res2))
    expect_equal(dim(res), c(2, 2))
    expect_equal(res2, c(16, 17, 18))
    expect_warning(findSignature(miniRAVmodel, keyword = "Bcell", k = 1),
                   "There is no RAV with 1 keyword-containing, enriched pathways.")
})

test_that("findKeywordInRAV function", {
    res <- findKeywordInRAV(miniRAVmodel, "Bcell", ind = 695)
    expect_equal(res, "1|2|3|4|5|6|8 (out of 9)")

    expect_error(findKeywordInRAV(miniRAVmodel, "Bcell", ind =  1),
                 "Selected ind \\(RAV1\\) doesn't exist.")
    expect_warning(findKeywordInRAV(miniRAVmodel, "hello", ind =  695),
                   "RAV695 doesn't have any pathway with the keyword, hello")
})

#' Plot heatmap of the sample scores
#'
#' @import ComplexHeatmap
#' @importFrom grid gpar
#'
#' @param score An output from \code{\link{calculateScore}} function, which is
#' a matrix with samples (row) and PrcompClusters (column) If it is a simple
#' vector, it will be converted to a one-column matrix.
#' @param dataName Title on the row. The name of the dataset to be scored.
#' @param modelName Title on the column. The RAVmodel used for scoring.
#' @param cluster_rows A logical. Under the default (\code{TRUE}), rows will be
#' clustered.
#' @param cluster_columns A logical. Under the default (\code{TRUE}), columns
#' will be clustered.
#' @param show_row_names Whether show row names. Default is \code{TRUE}, showing
#' the row name.
#' @param show_column_names Whether show column names. Default is \code{TRUE},
#' showing the column name.
#' @param row_names_gp Graphic parameters for row names. The default is 0.7.
#' @param column_names_gp Graphic parameters for column names. The default is 5.
#' @param ... Any additional argument for \code{\link[ComplexHeatmap]{Heatmap}}
#'
#' @return A heatmap of the sample score. Rows represent samples and columns
#' represent RAVs.
#'
#' @examples
#' data(miniRAVmodel)
#' library(bcellViper)
#' data(bcellViper)
#' score <- calculateScore(dset, miniRAVmodel)
#' sampleScoreHeatmap(score, dataName="bcellViper", modelName="miniRAVmodel")
#'
#' @export
sampleScoreHeatmap <- function(score, dataName, modelName,
                               cluster_rows = TRUE,
                               cluster_columns = TRUE,
                               show_row_names = TRUE,
                               show_column_names = TRUE,
                               row_names_gp = 0.7,
                               column_names_gp = 5, ...) {
    ComplexHeatmap::Heatmap(score,
                            cluster_rows = cluster_rows,
                            cluster_columns = cluster_columns,
                            row_title = dataName,
                            row_title_gp = gpar(fontsize = 11,
                                                fontface = "bold"),
                            column_title = modelName,
                            column_title_gp = gpar(fontsize = 11,
                                                   fontface = "bold"),
                            row_names_gp = gpar(fontsize = row_names_gp),
                            column_names_gp = gpar(fontsize = column_names_gp),
                            show_row_names = show_row_names,
                            show_column_names = show_column_names,
                            heatmap_width = unit(10, "cm"),
                            heatmap_legend_param = list(title = "score"), ...)
}

.heatmapTableMultiStudies <- function(val_all, RAVmodel,
                                      scoreCutoff = scoreCutoff,
                                      breaks = breaks, colors = colors,
                                      column_title = column_title,
                                      row_title = row_title,
                                      filterMessage = filterMessage, ...) {

  dat <- validatedSignatures(val_all, scoreCutoff = scoreCutoff,
                             filterMessage = FALSE)

  ## Define Heatmap Table Size
  if (is.null(column_title)) {
    column_title <- character(0)
    hmHeight <- nrow(dat) + 1
  } else {
    column_title <- column_title
    hmHeight <- nrow(dat) + 2
  }

  if (is.null(row_title)) {
    row_title <- character(0)
    hmWidth <- ncol(dat) + 1
  } else {
    row_title <- row_title
    hmWidth <- ncol(dat) + 2
  }

  ## Check the RAV quality
  dat_ind <- gsub("RAV", "", colnames(dat)) %>% as.numeric()
  .lowQualityRAVs(RAVmodel, dat_ind, filterMessage)

  ## Draw Heatmap
  ComplexHeatmap::Heatmap(dat, col = circlize::colorRamp2(breaks, colors),
                          name = "Corr",
                          cluster_rows = FALSE,
                          cluster_columns = FALSE,
                          cell_fun = function(j, i, x, y, width, height, fill) {
                            grid::grid.text(sprintf("%.2f", dat[i, j]), x, y,
                                            gp = grid::gpar(fontsize = 8))
                          },
                          show_heatmap_legend = FALSE,
                          row_title = row_title,
                          row_title_gp = grid::gpar(fontsize = 11,
                                                    fontface = "bold"),
                          row_names_gp = grid::gpar(fontsize = 10),
                          row_names_side = "right",
                          row_names_max_width = unit(0.5, "cm"),
                          column_title = column_title,
                          column_title_gp = grid::gpar(fontsize = 11,
                                                       fontface = "bold"),
                          column_names_side = "bottom",
                          column_names_gp = grid::gpar(fontsize = 10),
                          column_names_rot = 90,
                          column_names_max_height = unit(1, "cm"),
                          heatmap_width = unit(hmWidth, "cm"),
                          heatmap_height = unit(hmHeight, "cm"), ...
  )
}

#' Validation result in heatmap format
#'
#' This function subsets \code{\link{validate}} outputs with different criteria
#' and visualize it in a heatmap-like table.
#'
#' @param val_all An output matrix from \code{\link{validate}} function with the
#' parameter \code{level = "max"}. Subset of this matrix is plotted as a heatmap
#' using \code{\link[ComplexHeatmap]{Heatmap}}
#' @param RAVmodel PCAGenomicSignatures-class object. RAVmodel used to prepare
#' \code{val_all} input.
#' @param ind An integer vector. If this parameter is provided, the other
#' parameters, \code{num.out, scoreCutoff, swCutoff, clsizeCutoff} will be
#' ignored and the heatmap table containing only the provided index will be
#' printed.
#' @param num.out A number of highly validated RAVs to output. Default is 5.
#' If any of the cutoff parameters are provided, \code{num.out} or the number of
#' filtered RAVs, whichever smaller, will be chosen.
#' @param scoreCutoff A numeric value for the minimum correlation (not include).
#' If \code{val_all} input is from multiple studies, the default is 0.7 and this is
#' the only cutoff criteria considered: \code{swCutoff} and \code{clsizeCutoff}
#' will be ignored.
#' @param swCutoff A numeric value for the minimum average silhouette width.
#' @param clsizeCutoff A integer value for the minimum cluster size.
#' @param breaks A numeric vector of length 3. Number represents the values
#' assigned to three colors. Default is \code{c(0, 0.5, 1)}.
#' @param colors A character vector of length 3. Each represents the color
#' assigned to three breaks. Default is \code{c("white", "white smoke", "red")}.
#' @param column_title A character string. Provide the column title.
#' @param row_title A character string. Provide the row title.
#' @param whichPC An integer value between 1 and 8. PC number of your data to
#' check the validated signatures with. Under the default (\code{NULL}), it
#' outputs top scored signatures with any PC of your data.
#' @param ... any additional argument for \code{\link[ComplexHeatmap]{Heatmap}}
#' @param filterMessage A logical. Under the default \code{TRUE}, any output
#' RAV belong to the filtering list will give a message. Silence this message
#' with \code{filterMessage=FALSE}. You can check the filter list using
#' \code{data("filterList")}.
#'
#' @return A heatmap displaying the subset of the validation result that met the
#' given cutoff criteria. If \code{val_all} input is from a single dataset, the
#' output heatmap will contain both score and average silhouette width for each
#' cluster.
#'
#' If \code{val_all} input is from multiple studies, the output heatmap's rows
#' will represent each study and the columns will be RAVs, which meet
#' \code{scoreCutoff} for any of the input studies.
#'
#' @examples
#' data(miniRAVmodel)
#' library(bcellViper)
#' data(bcellViper)
#'
#' ## Single dataset
#' val_all <- validate(dset, miniRAVmodel)
#' heatmapTable(val_all, miniRAVmodel, swCutoff = 0)
#'
#' ## A list of datasets
#' val_all2 <- validate(miniTCGA, miniRAVmodel)
#' heatmapTable(val_all2, miniRAVmodel)
#'
#' @export
heatmapTable <- function(val_all, RAVmodel, ind = NULL, num.out = 5,
                         scoreCutoff = NULL, swCutoff = NULL,
                         clsizeCutoff = NULL,
                         breaks = c(0, 0.5, 1),
                         colors = c("white", "white smoke", "red"),
                         column_title = NULL, row_title = NULL,
                         whichPC = NULL, filterMessage = TRUE, ...) {

  score_ind <- which(colnames(val_all) == "score")
  sw_ind <- which(colnames(val_all) == "sw")

  ## If the validation result contains all PCs (`validate` with `level = "all"`)
  if (identical(colnames(val_all), paste0("PC", seq_len(8)))) {
    stop("'val_all' input should be created by `validate` function with
         `level = \"max\"`, not `level = \"all\"`.")
  }

  ## If the validation result is from the list of datasets
  if (length(score_ind) == 0) {
    res <- .heatmapTableMultiStudies(val_all, RAVmodel,
                                     scoreCutoff = scoreCutoff,
                                     breaks = breaks, colors = colors,
                                     column_title = column_title,
                                     row_title = row_title,
                                     filterMessage = filterMessage)
    return(res)
    stop
  }

  if (is.null(ind)) {
    val <- validatedSignatures(val_all, RAVmodel,
                               num.out = num.out, whichPC = whichPC,
                               indexOnly = FALSE, scoreCutoff = scoreCutoff,
                               swCutoff = swCutoff, clsizeCutoff = clsizeCutoff,
                               filterMessage = FALSE)
    dat <- val[,score_ind,drop=FALSE] %>% t
    sw <- val[,sw_ind,drop=FALSE] %>% as.numeric
  } else {
    row_ind <- which(rownames(val_all) %in% paste0("RAV", ind))
    val <- val_all[row_ind,]
    dat <- val[,score_ind,drop=FALSE] %>% t
    sw <- val[,sw_ind,drop=FALSE] %>% unlist %>% as.numeric
  }

  ## Define Heatmap Table Size
  if (is.null(column_title)) {
    column_title <- character(0)
    hmHeight <- nrow(dat) + 2.5
  } else {
    column_title <- column_title
    hmHeight <- nrow(dat) + 4
  }

  if (is.null(row_title)) {
    row_title <- character(0)
    hmWidth <- ncol(dat) + 2
  } else {
    row_title <- row_title
    hmWidth <- ncol(dat) + 3
  }

  ## Heatmap annotation representing Silhouette width of selected RAVs
  avg_sw <- ComplexHeatmap::anno_barplot(
    sw, baseline = 0, gp = grid::gpar(fill = ifelse(sw > 0, "red", "blue"),
                                      alpha = 0.5))
  gp <- grid::gpar(fontsize = 10)
  ha <- ComplexHeatmap::HeatmapAnnotation(avg.sw = avg_sw,
                                          annotation_name_side = "left",
                                          annotation_name_gp = gp)

  ## Check the RAV quality
  dat_ind <- gsub("RAV", "", colnames(dat)) %>% as.numeric()
  .lowQualityRAVs(RAVmodel, dat_ind, filterMessage)

  ## Draw Heatmap
  ComplexHeatmap::Heatmap(dat, col = circlize::colorRamp2(breaks, colors),
                          name = "Corr",
                          cluster_rows = FALSE,
                          cluster_columns = FALSE,
                          cell_fun = function(j, i, x, y, width, height, fill) {
                            grid::grid.text(sprintf("%.2f", dat[i, j]), x, y,
                                            gp = grid::gpar(fontsize = 8))
                          },
                          row_title = row_title,
                          row_title_gp = grid::gpar(fontsize = 11,
                                                    fontface = "bold"),
                          row_names_gp = grid::gpar(fontsize = 10),
                          row_names_side = "left",
                          row_names_max_width = unit(0.5, "cm"),
                          column_title = column_title,
                          column_title_gp = grid::gpar(fontsize = 11,
                                                       fontface = "bold"),
                          column_names_side = "bottom",
                          column_names_gp = grid::gpar(fontsize = 10),
                          column_names_rot = 90,
                          column_names_max_height = unit(1, "cm"),
                          heatmap_width = unit(hmWidth, "cm"),
                          heatmap_height = unit(hmHeight, "cm"),
                          top_annotation = ha, ...
  )
}

#' Extract information on a specific RAV
#'
#' @param RAVmodel A PCAGenomicSignatures object
#' @param ind An index of RAV
#' @return A list with four elements: clusterSize, silhouetteWidth,
#' enrichedPathways (the number of enriched pathways), and members.
#' The 'members' is the summary table of PCs in RAV, containing three columns:
#' studyName, PC, and Variance explained (%).
#'
#' @examples
#' data(miniRAVmodel)
#' getRAVInfo(miniRAVmodel, ind = 438)
#'
#' @export
getRAVInfo <- function(RAVmodel, ind) {

    rav <- paste0("RAV", ind)
    meta <- metadata(RAVmodel)
    colDat <- colData(RAVmodel)
    sw <- colDat$silhouetteWidth

    # PCs in the cluster
    PCmembership <- findStudiesInCluster(RAVmodel, ind)

    # metadata with study
    res <- vector(mode = "list")
    res$clusterSize <- as.numeric(meta$size[rav])
    res$silhouetteWidth <- round(as.numeric(sw[as.character(ind)]), 2)
    res$enrichedPathways <- nrow(colDat$gsea[[rav]])
    res$members <- PCmembership

    return(res)
}

#' Extract information on a specific training dataset
#'
#' @param RAVmodel A PCAGenomicSignatures object
#' @param study A character for SRA study accession.
#' @return A list with three elements: studyTitle, studySize (the number of
#' samples from this study used in the RAVmodel building), and RAVs. 'RAVs' is
#' a data frame with three columns - PC (1 to 20), RAV (RAV that the given PC
#' belongs to), and Variance explained (%). In the example, we used
#' miniRAVmodel, which doesn't have all the PCA summary information, so the
#' example will return only the two PCs of the study instead of all twenty.
#'
#' @examples
#' data(miniRAVmodel)
#' getStudyInfo(miniRAVmodel, "SRP028155")
#'
#' @export
getStudyInfo <- function(RAVmodel, study) {
    meta <- metadata(RAVmodel)
    trainingDat <- trainingData(RAVmodel)

    # Index of the requested study
    x <- gsub("\\.PC.*$", "", names(meta$cluster))
    ind <- which(x == study)

    # Variance explained by each PCs
    pcaRes <- trainingDat$PCAsummary[[study]]
    varExplained <- round(pcaRes["Variance",]*100, 2)

    # Clusters where 20 PCs are belong to
    metaCl <- meta$cluster[ind]
    PCnum <- as.numeric(gsub(".*\\.PC", "", names(metaCl)))

    PCdistribution <- data.frame(PC = PCnum,
                                 RAV = as.numeric(metaCl),
                                 Var = varExplained[PCnum],
                                 row.names = NULL)
    colnames(PCdistribution)[3] <- "Variance explained (%)"

    # Study info
    studyMeta <- .getStudyMeta(RAVmodel)
    studyMeta_ind <- which(studyMeta$studyName == study)
    studyTitle <- studyMeta$title[studyMeta_ind]  # title
    studySize <- studyMeta$downloaded[studyMeta_ind]  # size

    # Output
    res <- vector(mode = "list")
    res$studyTitle <- studyTitle
    res$studySize <- studySize
    res$RAVs <- PCdistribution
    return(res)
}

context("Test getRAVInfo and getStudyinfo functions")

data(miniRAVmodel)

test_that("getRAVInfo", {
    res <- getRAVInfo(miniRAVmodel, ind = 438)
    val <- c(5,7,9,3,5,6)

    expect_true(is.list(res))
    expect_equal(length(res), 4)
    expect_true(all(unlist(res[1:3]) == c(6, 0.04, 43)))
    expect_equal(res[[4]]$PC, val)
})

test_that("getStudyInfo", {
    res2 <- getStudyInfo(miniRAVmodel, "SRP119465")
    val2 <- c("studyTitle", "studySize", "RAVs")

    expect_equal(names(res2), val2)
    expect_equal(dim(res2$RAVs), c(3, 3))
    expect_equal(res2$studySize, 100)
})

#' Plot validation results in an interactive graph
#'
#' There are three main information on the graph:
#' \itemize{
#'     \item x-axis : Pearson correlation coefficient. Higher value means that
#'     test dataset and RAV is more tightly associated with.
#'     \item y-axis : Silhouette width representing the quality of RAVs.
#'     \item size : The number of studies in each RAV. (= cluster size)
#'     \item color : Test dataset's PC number that validate each RAV. Because we
#'     used top 8 PCs of the test dataset, there are 8 categories.
#' }
#'
#' @import ggplot2
#'
#' @param val_all Output from \link{validate} function.
#' @param swFilter If \code{swFilter=TRUE}, only RAV above the cutoff, defined
#' through \code{minSilhouetteWidth} argument will be plotted. Default is
#' \code{swFilter=FALSE}
#' @param minClusterSize The minimum size of clusters to be included in the
#' plotting. Default value is 2, so any single-element clusters are excluded.
#' @param minSilhouetteWidth A minimum average silhouette width to be plotted.
#' Only effective under \code{swFilter=TRUE} condition. Default is 0.
#' @param interactive If set to \code{TRUE}, the output will be interactive
#' plot. Default is \code{FALSE}.
#' @param minClSize The minimum number of PCs in the clusters you want.
#' @param maxClSize The maximum number of PCs in the clusters you want.
#' @param colorPalette Default is \code{Dark2}. For other color options, please
#' check \code{\link[ggplot2]{scale_color_brewer}}
#'
#' @return a ggplot object
#'
#' @examples
#' data(miniRAVmodel)
#' library(bcellViper)
#' data(bcellViper)
#' val_all <- validate(dset, miniRAVmodel)
#' plotValidate(val_all)
#'
#' @export
plotValidate <- function(val_all, minClusterSize = 2, swFilter = FALSE,
                         minSilhouetteWidth = 0, interactive = FALSE,
                         minClSize = NULL, maxClSize = NULL,
                         colorPalette = "Dark2") {

    # Validate inputs
    stopifnot(length(swFilter) == 1L, !is.na(swFilter), is.logical(swFilter))
    stopifnot(length(interactive) == 1L,
              !is.na(interactive), is.logical(interactive))

    # If the validation result contains all PCs (val_all from `level = "all"`)
    if (identical(colnames(val_all), paste0("PC", seq_len(8)))) {
        stop("'val_all' input should be created by `validate` function with
         `level = \"max\"`, not `level = \"all\"`.")
    }

    # If the validation result is from the list of datasets
    score_ind <- which(colnames(val_all) == "score")
    if (length(score_ind) == 0) {
        stop("'val_all' input should be from one dataset,
             not from multiple datasets")
    }

    ## Binding the variables from val_all locally to the function
    cl_size <- sw <- score <- cl_num <- PC <- NULL

    val_all <-  as.data.frame(val_all)
    clSizeFiltered <- dplyr::filter(val_all, cl_size >= minClusterSize)

    if (swFilter) {
        filtered <- dplyr::filter(clSizeFiltered, sw >= minSilhouetteWidth)
    } else {filtered <- clSizeFiltered}

    ind <- which(colnames(filtered) == "PC")
    filtered[,ind] <- factor(filtered[,ind])

    p <- ggplot(filtered, aes(sw, score,
                              size = cl_size,
                              label = cl_num,
                              color = PC)) +
        geom_point(alpha = 0.5) +
        scale_size(range = c(0.5, 8), name = "Cluster Size") +
        scale_color_brewer(palette = colorPalette) +
        theme_bw() +
        xlab("Average silhouette width of each cluster") +
        ylab("Validation score")

    if (interactive) {plotly::ggplotly(p)} else {p}
}

#' @name miniRAVmodel
#' @title RAVmodel from 536 studies, annotated with MSigDB C2
#' @docType data
#' @description
#' A object providing a miniature version of RAVmodel_C2 (PCAGenomicSignatures
#' object constructed from 536 studies and annotated with MSigDB C2).
#' @keywords data
#' @format PCAGenomicSignatures
#' @author Sehyun Oh \email{shbrief@gmail.com}
#'
"miniRAVmodel"


#' @name droplist
#' @title MeSH terms to be excluded in drawWordcloud function
#' @format A character vector containing MeSH terms to be excluded.
#' @author Sehyun Oh \email{shbrief@gmail.com}
"droplist"


#' @name filterList
#' @title RAVs that will output with quality-control messages
#' @format A named list with four elements - "Cluster_Size_filter",
#' "GSEA_C2_filter", "GSEA_PLIERpriors_filter", and "Redundancy_filter".
#' @author Sehyun Oh \email{shbrief@gmail.com}
"filterList"


#' @name miniAllZ
#' @title Subset of allZ matrix constructed from 8 CRC training datasets
#' @docType data
#' @description
#' Eight colorectal cancer microarray datasets were used to build RAVmodel and
#' the intermediate file containing genes and top PCs from each dataset is named
#' as \code{allZ}. Hierarchical clustering result of \code{allZ} is saved as
#' \code{res_hcut}. For demonstration, we subset the \code{allZ} matrix with the
#' first 100 genes, which is named as \code{miniAllZ}.
#' @keywords data
#' @format A matrix with 100 genes and 160 PCs from 8 training datasets.
#' @author Sehyun Oh \email{shbrief@gmail.com}
#' @source https://github.com/shbrief/model_building/tree/main/RAVmodel_8CRC
#'
"miniAllZ"

#' @name res_hcut
#' @title Subset of allZ matrix constructed from 8 CRC training datasets
#' @docType data
#' @description
#' Eight colorectal cancer microarray datasets were used to build RAVmodel and
#' the intermediate file containing genes and top PCs from each dataset is named
#' as \code{allZ}. Hierarchical clustering result of \code{allZ} is saved as
#' \code{res_hcut}.
#' @keywords data
#' @format \code{hclust} object from \code{factoextra::hcut} function.
#' @author Sehyun Oh \email{shbrief@gmail.com}
#'
"res_hcut"

#' @name miniTCGA
#' @title Subset of TCGA-COAD and TCGA-BRCA RNA sequencing datasets
#' @docType data
#' @description
#' TCGA-COAD and TCGA-BRCA RNA sequencing data were acquired using
#' \code{GSEABenchmarkeR::loadEData} and log-transformed. Conversion from
#' EntrezID to gene symbol was done with \code{EnrichmentBrowser::idMap}. Only
#' 8 samples from each dataset are kept.
#' @keywords data
#' @format A list containing two SummarizedExperiment objects.
#' @author Sehyun Oh \email{shbrief@gmail.com}
#'
"miniTCGA"

#' Extract the list of PCs in a cluster
#'
#' A RAV model contain clusters of PCs from individual
#' studies. This function extracts the names of the original
#' PCs from the RAV model given the index in the RAV model.
#'
#' @param RAVmodel A PCAGenomicSignatures object
#' @param ind An index of RAV
#'
#' @return A character vector of PC/study names
#'
#' @examples
#' data(miniRAVmodel)
#' PCinRAV(miniRAVmodel,695)
#'
#' @export
PCinRAV <- function(RAVmodel, ind) {

    ## Switching metadata structure of RAVmodel
    if (version(RAVmodel) == ">= 0.0.7" | version(RAVmodel) == ">= 0.0.6") {
        cluster <- S4Vectors::metadata(RAVmodel)$cluster
        k <- which(cluster == ind)
        out <- names(k)
    } else {
        ##### update with RAVmodel restructure #####
        # cluster <- colData(RAVmodel)$cluster
        # out <- cluster[[paste0("RAV", ind)]]
        cluster <- S4Vectors::metadata(RAVmodel)$cluster
        k <- which(cluster == ind)
        out <- names(k)
    }

    return(out)
}


#' Build a two-column word/frequency table
#'
#' @import dplyr
#' @importFrom S4Vectors metadata
#'
#' @param RAVmodel A PCAGenomicSignatures object
#' @param ind An index of RAV
#' @param rm.noise An integer. Under the default (\code{rm.noise=NULL}), if
#' cluster size (= \code{s}) is smaller than 8, \code{rm.noise = floor(s*0.5)}.
#' For clusters with >= 8 PCs, \code{rm.noise = 4}. If \code{rm.noise = 0}, all
#' the MeSH terms in RAV will be used to draw wordcloud.
#' @param weighted A logical. If \code{TRUE}, MeSH terms from each study are
#' weighted based on the variance explained by the principle component of the
#' study contributing a give RAV. Default is \code{TRUE}.
#' @param filterMessage A logical. Under the default \code{TRUE}, any output
#' RAV belong to the filtering list will give a message. Silence this message
#' with \code{filterMessage=FALSE}. You can check the filter list using
#' \code{data("filterList")}.
#'
#' @return A table with two columns, \code{word} and \code{freq}. MeSH terms in
#' the defined RAV (by \code{ind} argument) is ordered based on their frequency.
#'
#' @examples
#' data(miniRAVmodel)
#' meshTable(miniRAVmodel,1139)
#'
#' @export
meshTable <- function(RAVmodel, ind, rm.noise = NULL,
                      weighted = TRUE, filterMessage = TRUE) {
    
    ### Check ind validity for meshTable
    if (length(ind) != 1) {stop("Provide only one index.")}
    if (ind %% 1 != 0) {stop("`ind` should be integer.")}

    ### Check ind validity
    .availableRAV(RAVmodel, ind)

    ### Check the RAV quality
    .lowQualityRAVs(RAVmodel, ind, filterMessage)

    ### Remove noise
    if (is.null(rm.noise)) {
        s <- S4Vectors::metadata(RAVmodel)$size[paste0("RAV", ind)]
        # s <- colData(RAVmodel)$size[paste0("RAV", ind)] ## update with RAVmodel restructure
        if (s < 8) {rm.noise <- floor(s*0.5)}
        else if (s >= 8) {rm.noise <- 4}
    }

    ### Create a 'universe' for bag-of-words model
    # frequency of the `name` in the background
    bow <- unlist(S4Vectors::metadata(RAVmodel)$MeSH_freq)
    # remove rare terms
    bow <- bow[which(bow > rm.noise)]

    ### Not weighted version
    if (weighted == FALSE) {
        ind_name <- paste0("Cl", metadata(RAVmodel)$k, "_", ind)
        study_id <- studies(RAVmodel)[[ind_name]]   # a list of studies in RAV
        all_MeSH <- mesh(RAVmodel)   # all the MeSH data

        # remove SRP069088 (no MeSH term)
        if ("SRP069088" %in% study_id) {
            ind_rm <- which(study_id == "SRP069088")
            PCs <- PCs[-ind_rm]
            var <- var[,-ind_rm]
            study_id <- study_id[-ind_rm]
        }

        # subset to the participating studies
        mesh_subset <- all_MeSH[study_id]

        ### Combine all MeSH words
        d <- list()
        for (i in seq_along(mesh_subset)) {
            d <- c(d, mesh_subset[[i]]$name)
        }

        ### Build a term-frequency table
        summary <- unlist(d)
        summary <- table(summary)
        summary <- summary[names(summary) %in% names(bow)]

    ### Weighted based on variance explained by PC
    } else {
        PCs <- PCinRAV(RAVmodel, ind)
        varAll <- Reduce(cbind, PCAsummary(RAVmodel))
        var <- varAll[,PCs,drop=FALSE]
        study_id <- gsub("\\.PC.*$", "", PCs)
        all_MeSH <- mesh(RAVmodel)   # all the MeSH data

        # remove SRP069088 (no MeSH term)
        if ("SRP069088" %in% study_id) {
            ind_rm <- which(study_id == "SRP069088")
            PCs <- PCs[-ind_rm]
            var <- var[,-ind_rm]
            study_id <- study_id[-ind_rm]
        }

        mesh_subset <- sapply(all_MeSH[study_id], function(x) {x$name})

        weight <- as.data.frame(matrix(nrow = 0, ncol = 3))
        colnames(weight) <- c("PCs", "var", "mesh")

        for (i in seq_along(PCs)) {
            new <- data.frame(PCs = PCs[i],
                             var = var["Variance", i],
                             mesh = mesh_subset[[i]])
            weight <- rbind(weight, new)
        }

        weight <- weight %>% group_by(mesh) %>% summarise(total_var = sum(var))

        ### Build a term-frequency table
        summary <- stats::setNames(weight$total_var, as.character(weight$mesh))
        summary <- summary[names(summary) %in% names(bow)]
    }

    for (i in seq_along(summary)) {
        summary[i] <- summary[i]/bow[names(summary[i])]
    }
    all <- as.data.frame(matrix(NA, ncol = 0, nrow = length(summary)))
    all$word <- names(summary)
    all$freq <- summary
    all <- all[order(all$freq, decreasing = TRUE),]

    return(all)
}


#' @title Draw wordcloud using the collection of RAVs' MeSH terms
#' @description Plot a word cloud using the remaining MeSH terms in the selected
#' RAV after user-defined filtering.
#'
#' @param RAVmodel PCAGenomicSignatures object
#' @param ind An index of the RAV you want to draw wordcloud.
#' @param rm.noise An integer. Under the default (\code{rm.noise=NULL}), if
#' cluster size (= \code{s}) is smaller than 8, \code{rm.noise = floor(s*0.5)}.
#' For clusters with >= 8 PCs, \code{rm.noise = 4}. If \code{rm.noise = 0}, all
#' the MeSH terms in RAV will be used to draw wordcloud.
#' @param scale A \code{scale} argument for \code{\link[wordcloud]{wordcloud}}
#' function
#' @param weighted A logical. If \code{TRUE} (default), MeSH terms from each
#' study are weighted based on the variance explained by the principle component
#' of the study contributing to a given RAV.
#' @param drop A character vector containing MeSH terms to be excluded from word
#' cloud. Under the default (\code{NULL}), manually selected non-informative
#' MeSH terms are excluded, which can be viewed through \code{data(droplist)}.
#' @param filterMessage A logical. Under the default \code{TRUE}, any output
#' RAV belong to the filtering list will give a message. Silence this message
#' with \code{filterMessage=FALSE}. You can check the filter list using
#' \code{data("filterList")}.
#'
#' @return A word cloud with the MeSH terms associated with the given cluster.
#'
#' @examples
#' data(miniRAVmodel)
#' drawWordcloud(miniRAVmodel, 1139)
#'
#' @export
drawWordcloud <- function(RAVmodel, ind, rm.noise = NULL, scale = c(3, 0.5),
                          weighted = TRUE, drop = NULL, filterMessage = TRUE) {

    # Check ind validity
    .availableRAV(RAVmodel, ind)

    # Check the RAV quality
    .lowQualityRAVs(RAVmodel, ind, filterMessage)

    if (is.null(rm.noise)) {
        s <- S4Vectors::metadata(RAVmodel)$size[paste0("RAV", ind)]
        # s <- colData(RAVmodel)$size[paste0("RAV", ind)] ## update with RAVmodel restructure
        if (s < 8) {rm.noise <- floor(s*0.5)}
        else if (s >= 8) {rm.noise <- 4}
    }

    # MeSH word table
    all <- meshTable(RAVmodel, ind, rm.noise = rm.noise,
                     weighted = weighted, filterMessage = FALSE)

    # Remove enriched MeSH term if it is in 'droplist'
    if (!is.null(drop)) {
        droplist <- drop
    } else {
        local_data_store <- new.env(parent = emptyenv())
        data("droplist", envir = local_data_store, package = "GenomicSuperSignature")
        droplist <- local_data_store[["droplist"]]
    }
    drop_ind <- which(all$word %in% droplist)
    if (length(drop_ind) != 0) {all <- all[-drop_ind,]}
    if (nrow(all) == 0) {stop("No MeSH term is enriched.")}

    # generate the word cloud
    wordcloud::wordcloud(words = all$word, freq = all$freq, scale = scale,
                         max.words = Inf, random.order = FALSE, rot.per = 0,
                         colors = RColorBrewer::brewer.pal(8, "Dark2"))
}

data(miniRAVmodel)

test_that("meshTable function", {
    res <- meshTable(miniRAVmodel, 1139)
    expect_equal(res[1,1], "NOTCH1 protein, human")
    expect_equal(round(res[1,2], 4), 0.1242)

    res2 <- meshTable(miniRAVmodel, 1139, weighted = FALSE)
    expect_equal(res2[1,1], "Muromonab-CD3")
    expect_equal(round(res2[1,2], 4), 0.3333)

    res3 <- meshTable(miniRAVmodel, 1139, weighted = FALSE, rm.noise = 5)
    expect_equal(res3[1,1], "Infant")
    expect_equal(round(res3[1,2], 4), 0.1429)

    expect_error(meshTable(miniRAVmodel, 1),
                 "Selected ind \\(RAV1\\) doesn't exist.")
    expect_equal(dim(meshTable(miniRAVmodel, 1994)), c(31, 2))
})

expect_error(drawWordcloud(miniRAVmodel, 1),
             "Selected ind \\(RAV1\\) doesn't exist.")

#' @include GenomicSignatures-class.R


#' @title Formatting RAV name
#'
#' @description Keep the name with 'k + cluster number + number of PCs + number
#' of unique studies' info during the model construction to make it easy to keep
#' track of them, but at the PCAGenomicSignatures-class object building step,
#' covert them into 'RAV + cluster number'.
#'
#' @param x PCAGenomicSignatures object
#' @param ... Additional arguments for supporting functions.
#'
#' @return a character vector
#' 
#' @keywords internal
#'
.RAVName <- function(x, ...) {
    rownames(x@colData) <- paste0("RAV", seq_len(ncol(x)))
    x@colData$RAV <- rownames(x@colData)
    return(x)
}


### ==============================================
### PCAGenomicSignatures Constructor
### ==============================================
#' @name PCAGenomicSignatures
#' @title Construct \code{PCAGenomicSignatures} object
#'
#' @description The default contents of \code{PCAGenomicSignatures} object, with
#' a set of accessors and setter generic functions, which extract either the
#' \code{assay}, \code{colData}, \code{metadata}, or \code{trainingData} slots
#' of a \code{\link{PCAGenomicSignatures-class}} object. When you create this
#' object, \code{colData$studies} should be populated before adding any
#' information in \code{trainingData} slot
#'
#' @details
#' \itemize{
#'     \item RAVindex(x) : RAVindex (= avgLoadings) containing genes x RAVs
#'     \item metadata(x)$cluster : A vector of integers (from 1:k) indicating
#'     the cluster to which each point is allocated.
#'     \item metadata(x)$size : The number of PCs in each cluster.
#'     \item metadata(x)$k : The number of RAVs.
#'     \item metadata(x)$n : The number of top PCs from each dataset.
#'     \item metadata(x)$geneSets : Name of the prior gene sets used to annotate
#'     average loadings.
#'     \item colData(x)$studies : A list of character vectors containing studies
#'     contributing to each PC cluster.
#'     \item colData(x)$silhouetteWidth : A numeric array of average silhouette
#'     widths of each clusters
#'     \item colData(x)$gsea : A list of data frames. Each element is a subset
#'     of outputs from \code{clusterProfiler::GSEA} function.
#' }
#'
#' @section Setters:
#' Setter method values (i.e., \code{function(x) <- value}):
#' \itemize{
#'     \item geneSets<- : A character vector containing the name of gene sets
#'     used to annotate average loadings
#'     \item studies<- : A list of character vectors containing gene sets used
#'     to annotate average loadings
#'     \item gsea<- : A list of data frames. Each element is a subset of output
#'     from \code{gseaResult} objects.
#'     \item metadata<- : A \code{list} object of metadata
#'     \item `$<-` : A vector to replace the indicated column in \code{colData}
#' }
#'
#' @section Accessors:
#' All the accessors inherited from \code{SummarizedExperiment} are available
#' and the additional accessors for \code{PCAGenomicSignatures} specific data
#' are listed below.
#' \itemize{
#'    \item RAVindex : Equivalent to the \code{assay(x)}
#'    \item geneSets : Access the \code{metadata(x)$geneSets} slot
#'    \item studies : Access the \code{colData(x)$studies} slot
#'    \item gsea : Access the \code{colData(x)$gsea}
#'    \item `$` : Access a column in \code{colData}
#'    \item trainingData : Access the \code{trainingData} slot
#'    \item mesh : Access the \code{trainingData(x)$MeSH} slot
#'    \item PCAsummary : Access the \code{trainingData(x)$PCAsummary} slot
#' }
#'
#' @slot trainingData A \code{\link[S4Vectors]{DataFrame}} class object for
#' metadata associated with training data
#'
#' @param trainingData A \code{\link[S4Vectors]{DataFrame}} class object for
#' metadata associated with training data
#' @param ... Additional arguments for supporting functions.
#' @return PCAGenomicSignatures object with multiple setters or accessors
#'
#' @examples
#' data(miniRAVmodel)
#' miniRAVmodel
#'
#' @export
PCAGenomicSignatures <- function(..., trainingData)
{
    se <- SummarizedExperiment::SummarizedExperiment(...)
    .RAVName(se) #--> Move this to the model_building process instead
    gs <- .PCAGenomicSignatures(se, trainingData = trainingData)
}


#' @name PCAGenomicSignatures-methods
#' @title Methods and accesors for \code{PCAGenomicSignatures} object
#'
#' @description The default contents of \code{PCAGenomicSignatures} object, with
#' a set of accessor and setter generic functions, which extract either the
#' \code{assay}, \code{colData}, \code{metadata}, or \code{trainingData} slots
#' of a \code{\link{PCAGenomicSignatures-class}} object. When you create this
#' object, \code{colData$studies} should be populated before adding any
#' information in \code{trainingData} slot
#'
#' @details
#' \itemize{
#'     \item RAVindex(x) : RAVindex (= avgLoadings) containing genes x RAVs
#'     \item metadata(x)$cluster : A vector of integers (from 1:k) indicating
#'     the cluster to which each PC is allocated.
#'     \item metadata(x)$size : The number of PCs in each cluster.
#'     \item metadata(x)$k : The number of RAVs.
#'     \item metadata(x)$n : The number of top PCs from each dataset.
#'     \item metadata(x)$geneSets : Name of the prior gene sets used to annotate
#'     average loadings.
#'     \item colData(x)$studies : A list of character vectors containing studies
#'     contributing to each PC cluster.
#'     \item colData(x)$gsea : A list of data frames. Each element is a subset
#'     of outputs from \code{clusterProfiler::GSEA} function.
#' }
#'
#' @section Setters:
#' Setter method values (i.e., \code{function(x) <- value}):
#' \itemize{
#'     \item geneSets<- : A character vector containing the name of gene sets
#'     used to annotate average loadings
#'     \item studies<- : A list of character vectors containing gene sets used
#'     to annotate average loadings
#'     \item gsea<- : A list of \code{gseaResult} objects.
#'     \item metadata<- : A \code{list} object of metadata
#'     \item `$<-` : A vector to replace the indicated column in \code{colData}
#' }
#'
#' @section Accessors:
#' All the accessors inherited from \code{SummarizedExperiment} are available
#' and the additional accessors for \code{PCAGenomicSignatures} specific data
#' are listed below.
#' \itemize{
#'    \item RAVindex : Equivalent to the \code{assay(x)}
#'    \item geneSets : Access the \code{metadata(x)$geneSets} slot
#'    \item studies : Access the \code{colData(x)$studies} slot
#'    \item gsea : Access the \code{colData(x)$gsea}
#'    \item `$` : Access a column in \code{colData}
#'    \item trainingData : Access the \code{trainingData} slot
#'    \item mesh : Access the \code{trainingData(x)$MeSH} slot
#'    \item PCAsummary : Access the \code{trainingData(x)$PCAsummary} slot
#' }
#'
#' @slot trainingData A \code{\link[S4Vectors]{DataFrame}} class object for
#' metadata associated with training data
#'
#' @param object,x A \code{PCAGenomicSignatures} object
#' @param value See details.
#'
#' @examples
#' data(miniRAVmodel)
#' miniRAVmodel
#'
#' @return PCAGenomicSignatures object with multiple setters or accessors
#' @aliases studies<- silhouetteWidth<- gsea<- trainingData<- mesh<-
#' PCAsummary<- studies silhouetteWidth gsea trainingData mesh PCAsummary
NULL

### ==============================================
### Setter
### ==============================================
#' @export
setGeneric("studies<-", function(x, value) standardGeneric("studies<-"))

#' @exportMethod studies<-
#' @rdname PCAGenomicSignatures-methods
setMethod("studies<-", "PCAGenomicSignatures", function(x, value) {
    x@colData$studies <- value
    allStudies <- unlist(value)
    allStudies <- unique(allStudies)
    
    if (!all(allStudies %in% row.names(x@trainingData))) {
        stop("Studies do not match with the training datasets.")
    }
    # x@trainingData <- S4Vectors::DataFrame(row.names = allStudies) 
    # validObject(x)
    return(x)
})


#' @export
setGeneric("silhouetteWidth<-",
           function(x, value) standardGeneric("silhouetteWidth<-"))

#' @exportMethod silhouetteWidth<-
#' @rdname PCAGenomicSignatures-methods
setMethod("silhouetteWidth<-", "PCAGenomicSignatures", function(x, value) {
    x@colData$silhouetteWidth <- value
    return(x)
})


#' @export
setGeneric("gsea<-", function(x, value) standardGeneric("gsea<-"))

#' @exportMethod gsea<-
#' @rdname PCAGenomicSignatures-methods
setMethod("gsea<-", "PCAGenomicSignatures", function(x, value) {
    x@colData$gsea <- value
    return(x)
})


#' @export
setGeneric("trainingData<-",
           function(x, value) standardGeneric("trainingData<-"))

#' @exportMethod trainingData<-
#' @rdname PCAGenomicSignatures-methods
setMethod("trainingData<-", "PCAGenomicSignatures", function(x, value) {
    x@trainingData <- value
    # validObject(x)
    return(x)
})


#' @export
setGeneric("mesh<-", function(x, value) standardGeneric("mesh<-"))

#' @exportMethod mesh<-
#' @rdname PCAGenomicSignatures-methods
setMethod("mesh<-", "PCAGenomicSignatures", function(x, value) {
    trainingData(x)$MeSH <- NA
    for (i in seq_along(value)) {
        ind <- which(rownames(trainingData(x)) == names(value[i]))
        trainingData(x)$MeSH[ind] <- value[i]
    }
    names(trainingData(x)$MeSH) <- rownames(trainingData(x))
    return(x)
})


#' @export
setGeneric("PCAsummary<-", function(x, value) standardGeneric("PCAsummary<-"))

#' @exportMethod PCAsummary<-
#' @rdname PCAGenomicSignatures-methods
setMethod("PCAsummary<-", "PCAGenomicSignatures", function(x, value) {
    trainingData(x)$PCAsummary <- NA
    for (i in seq_along(value)) {
        ind <- which(rownames(trainingData(x)) == names(value[i]))
        trainingData(x)$PCAsummary[ind] <- value[i]
    }
    names(trainingData(x)$PCAsummary) <- rownames(trainingData(x))
    return(x)
})



### ==============================================
### Getter
### ==============================================
#' @export
setGeneric("studies", function(x) standardGeneric("studies"))

#' @exportMethod studies
#' @rdname PCAGenomicSignatures-methods
setMethod("studies", "PCAGenomicSignatures", function(x) {
    out <- x@colData$studies
    return(out)
})


#' @export
setGeneric("silhouetteWidth", function(x) standardGeneric("silhouetteWidth"))

#' @exportMethod silhouetteWidth
#' @rdname PCAGenomicSignatures-methods
setMethod("silhouetteWidth", "PCAGenomicSignatures", function(x) {
    out <- x@colData$silhouetteWidth
    return(out)
})


#' @export
setGeneric("gsea", function(x) standardGeneric("gsea"))

#' @exportMethod gsea
#' @rdname PCAGenomicSignatures-methods
setMethod("gsea", "PCAGenomicSignatures", function(x) {
    out <- x@colData$gsea
    return(out)
})


#' @export
setGeneric("trainingData", function(x) standardGeneric("trainingData"))

#' @exportMethod trainingData
#' @rdname PCAGenomicSignatures-methods
setMethod("trainingData", "PCAGenomicSignatures", function(x) {
    out <- x@trainingData
    return(out)
})


#' @export
setGeneric("mesh", function(x) standardGeneric("mesh"))

#' @exportMethod mesh
#' @rdname PCAGenomicSignatures-methods
setMethod("mesh", "PCAGenomicSignatures", function(x) {
    out <- x@trainingData$MeSH
    return(out)
})


#' @export
setGeneric("PCAsummary", function(x) standardGeneric("PCAsummary"))

#' @exportMethod PCAsummary
#' @rdname PCAGenomicSignatures-methods
setMethod("PCAsummary", "PCAGenomicSignatures", function(x) {
    out <- x@trainingData$PCAsummary
    return(out)
})



### ==============================================
### Show method
### ==============================================
#' @exportMethod show
#' @rdname PCAGenomicSignatures-methods
setMethod("show", "PCAGenomicSignatures", function(object) {
    callNextMethod()

    colnames <- colnames(trainingData(object))
    S4Vectors::coolcat("trainingData(%d): %s\n", colnames)

    rownames <- rownames(trainingData(object))
    S4Vectors::coolcat("trainingData names(%d): %s\n", rownames)
})

#' Find the studies contributing each RAV
#'
#' @param RAVmodel PCAGenomicSignatures object.
#' @param ind A numeric vector containing the RAV indexes. Under the default
#' (\code{NULL}), studies associated with all the RAV indexes will be returned
#' as a list.
#' @param studyTitle Default is \code{FALSE}. This parameter is effective only when
#' the \code{index} value is specificed. If it's \code{TRUE}, the output will be
#' a data frame with the study
#'
#' @return A list of character vectors. Under the default condition
#' (\code{ind = NULL}), all the RAVs will be checked for their contributing
#' studies and the length of the list will be same as the number of RAVs
#' (= \code{metadata(x)$k}). If you provide the \code{ind} argument, studies
#' associated with only the specified RAVs will be returned.
#'
#' @examples
#' data(miniRAVmodel)
#' findStudiesInCluster(miniRAVmodel, 1076)
#'
#' @note Mainly used for model building, within \link{buildAvgLoading}.
#' @export
findStudiesInCluster <- function(RAVmodel, ind = NULL, studyTitle = FALSE) {

  ## Check the input validity
  if (!is.null(ind)) {.availableRAV(RAVmodel, ind)}

  ## Extract cluster information: size and the number of PCs
  if (is(RAVmodel,"PCAGenomicSignatures")) {
    x <- S4Vectors::metadata(RAVmodel)
    k <- x$k   # the number of clusters
  } else if (is.list(RAVmodel)) {   # [internal] this is for model building
    x <- RAVmodel
    x$size <- table(RAVmodel$cluster)
    k <- length(unique(RAVmodel$cluster))
  }

  ## z is the binary matrix showing the cluster membership of PCs
  z <- matrix(0, ncol = k, nrow = sum(x$size))
  for (i in seq_along(x$cluster)) {
    z[i, x$cluster[i]] <- 1
  }
  colnames(z) <- paste0("Cl", k, "_",
                        formatC(seq_len(k), width=2, format="d", flag="0"))
  rownames(z) <- names(x$cluster)

  ## Extract study accession number part from PC names in each cluster
  if (is.null(ind)) {
    studies <- list()
    for (i in seq_len(ncol(z))) {
      studies[[i]] <- rownames(z)[which(z[,i] == 1)]
      studies[[i]] <- lapply(studies[[i]],
                             function(x) {unlist(strsplit(x, "\\.PC"))[1]})
      studies[[i]] <- unique(unlist(studies[[i]]))  # remove redundancy
      names(studies)[i] <- colnames(z)[i]
    }
  } else {
    studies <- .varByPCsInCluster(RAVmodel, ind)
  }

  ## Extract study titles
  if (!studyTitle) {
    res <- studies
  } else {
    studyMeta <- .getStudyMeta(RAVmodel)
    studyMeta <- studyMeta[,c("studyName", "title")]
    res <- dplyr::left_join(studies, studyMeta, by = "studyName")
  }

  return(res)
}

context("Test findStudiesInCluster function")

data(miniRAVmodel)

test_that("studyTitle argument", {
    res <- findStudiesInCluster(miniRAVmodel, 1076)
    res2 <- findStudiesInCluster(miniRAVmodel, 1076, studyTitle = TRUE)

    expect_true(is.data.frame(res))
    expect_true(is.data.frame(res2))
    expect_equal(dim(res2), c(10,4))
    expect_equal(res$studyName[1:3], c("SRP028155", "SRP049340", "SRP058840"))
})

test_that("only valid index", {
    expect_error(findStudiesInCluster(miniRAVmodel, 1),
                 "Selected ind \\(RAV1\\) doesn't exist.")
    expect_error(findStudiesInCluster(miniRAVmodel, c(1, 1076, 100)),
                 "Selected ind \\(RAV1, RAV100\\) doesn't exist.")
})

.validateScore <- function(dat, rescale.after, avg.loadings) {
    count <- .extractExprsMatrix(dat)
    count <- count[apply(count, 1,
                         function(x) {!any(is.na(x) | (x==Inf) | (x==-Inf))}),]
    count <- apply(count, 1, function(x) {x - mean(x)}) %>% t  # row centered
    gene_common <- intersect(rownames(avg.loadings), rownames(count))

    if (rescale.after) {
        # CRC paper version
        score <- t(count[gene_common,])%*%as.matrix(avg.loadings[gene_common,])
        score <- (t(score) / apply(score, 2, stats::sd)) %>% t
    } else {
        score <- t(count[gene_common,]) %*%
            apply(avg.loadings[gene_common,], 2,
                  function(x) x / sqrt(sum(x^2, na.rm = TRUE)))
    }

    colnames(score) <- colnames(avg.loadings)
    return(score)
}

#' Calculate the validation score for a new dataset
#'
#' @import methods
#'
#' @param dataset A gene expression profile to be validated. Different classes
#' of objects can be used including ExpressionSet, SummarizedExperiment,
#' RangedSummarizedExperiment, or matrix. Rownames (genes) should be in symbol
#' format. If it is a matrix, genes should be in rows and samples in columns.
#' @param RAVmodel PCAGenomicSignatures object. A matrix of average loadings, an
#' output from \code{buildAvgLoading}, can be directly provided.
#' @param rescale.after Under the default (\code{TRUE}), the continuous scores
#' are rescaled post assignment, so average loadings have the same standard
#' deviation in different studies. If it is \code{FALSE}, the rescaling of
#' column (= dividing by \code{sqrt(sum(x^2}) is done before score assignment.
#'
#' @return A list containing the score matrices for input datasets. Scores are
#' assigned to each sample (row) for each cluster (column).
#'
#' @examples
#' data(miniRAVmodel)
#' library(bcellViper)
#' data(bcellViper)
#' score <- calculateScore(dset, miniRAVmodel)
#'
#' data(miniTCGA)
#' score <- calculateScore(miniTCGA, miniRAVmodel)
#' 
#' @export
calculateScore <- function(dataset, RAVmodel, rescale.after = TRUE) {

    if (is(RAVmodel, "PCAGenomicSignatures")) {
        avg.loadings <- assay(RAVmodel)
    } else if (is.data.frame(RAVmodel) | is.matrix(RAVmodel)) {
        avg.loadings <- RAVmodel
    }

    if (!is.list(dataset)) {
        validationScore <- .validateScore(dataset, rescale.after, avg.loadings)
    } else {
        validationScore <- lapply(dataset, .validateScore,
                                  rescale.after, avg.loadings)
    }

    return(validationScore)
}

data(miniRAVmodel)
data(miniTCGA)
library(bcellViper)
data(bcellViper)

res <- calculateScore(dset, miniRAVmodel)
res2 <- calculateScore(miniTCGA, miniRAVmodel)
res3 <- calculateScore(miniTCGA, miniRAVmodel, rescale.after = FALSE)


test_that("check both input dataset formats works", {
    expect_true(is.matrix(res))
    expect_true(is.list(res2))
})

test_that("check whether `rescale.after` argument works", {
    expect_equal(round(res2[[1]][1,1], 4), -0.7251)
    expect_equal(round(res3[[1]][1,1], 4), -30.822)
})

.set_n <- function(dataset, n) {
    if (is.null(n)) {
      if (ncol(dataset) >= 8) {
        n = 8
      } else {
        n = ceiling(ncol(dataset)*0.5)
      }
    }
    return(n)
}


#' Validating new dataset
#'
#' @importFrom irlba irlba
#' 
#' @param dataset A gene expression profile to be validated. Different classes
#' of objects can be used including ExpressionSet, SummarizedExperiment,
#' RangedSummarizedExperiment, or matrix. Rownames (genes) should be in 
#' human gene symbol format (HGNC). If dataset is a matrix, genes should be in 
#' rows and samples in columns. RNA-seq counts should be log(count + 1) 
#' prior to the `validate()` call.
#' @param avgLoading A matrix with genes by RAVs.
#' @param n A integer. The number of PCs to use for validation. It should be
#' equal or less then the number of samples in the input dataset. 
#' @param method A character string indicating which correlation coefficient is
#' to be computed. One of "pearson" (default), "kendall", or "spearman": can be
#' abbreviated.
#' @param scale Default is \code{FALSE}. If it is set to \code{TRUE}, rows 
#' will be converted to z-score prior to PCA. 
#'
#' @return A matrix of Pearson correlation coefficient (default, defined through
#' \code{method} argument) between RAVs (row) and the top 8 PCs from the
#' datasets (column)
#' 
#' @keywords internal
#'
.loadingCor <- function(dataset, 
                        avgLoading,
                        n = n,
                        method = method, 
                        scale = FALSE) {

    # Extract expression matrix from different classes
    dat <- .extractExprsMatrix(dataset)

    # # row normalization # Plan to remove `scale` argument <<<<<<<<<<<<<<<<<<<< 
    # stopifnot(length(scale) == 1L, !is.na(scale), is.logical(scale))
    # if (scale) {dat <- t(scale(t(dat)))}

    if (!is(dataset, "SingleCellExperiment")) {
        dat <- dat[apply(dat, 1,
                         function (x) {!any(is.na(x) | (x==Inf) | (x==-Inf))}),]
        gene_common <- intersect(rownames(avgLoading), rownames(dat))
        prcomRes <- stats::prcomp(t(dat[gene_common,]))  # centered, but not scaled
        loadings <- prcomRes$rotation[, seq_len(n)]
    } else {    # Dimensional reduction for scRNAseq data
        gene_common <- intersect(rownames(avgLoading), rownames(dat))
        res <- irlba::irlba(as.matrix(t(dat[gene_common,])), 
                            nv = n, 
                            center = TRUE, scale = FALSE)
        loadings <- res$v
        rownames(loadings) <- rownames(dat[gene_common,])
        colnames(loadings) <- paste0("PC", seq_len(n))
    }
    
    loading_cor <- abs(stats::cor(avgLoading[gene_common,],
                                  loadings[gene_common,],
                                  use = "pairwise.complete.obs",
                                  method = method))
    return(loading_cor)
}


#' Validate new datasets
#'
#' @param dataset Single or a named list of SummarizedExperiment
#' (RangedSummarizedExperiment, ExpressionSet or matrix) object(s). Columns 
#' should contain samples (or cells for single-cell data), and the row names
#' should be in 'gene symbol' format. 
#' @param RAVmodel PCAGenomicSignatures object.
#' @param method A character string indicating which correlation coefficient is
#' to be computed. One of "pearson" (default), "kendall", or "spearman": can be
#' abbreviated.
#' @param maxFrom Select whether to display the maximum value from dataset's PCs
#' or avgLoadings. Under the default (\code{maxFrom="PC"}), the maximum
#' correlation coefficient from top 8 PCs for each avgLoading will be selected
#' as an output. If you choose (\code{maxFrom="avgLoading"}), the avgLoading
#' with the maximum correlation coefficient with each PC will be in the output.
#' @param n A integer. The number of PCs to use for validation. It should be
#' equal or less then the number of samples in the input dataset. If there are 
#' >= 8 samples in the input dataset or the input datasets is a list, it is set 
#' to 8 as a default. If there are less than 8 samples in the input, it is set 
#' to half of the number of samples as a default.
#' @param level Output format of validated result. Two options are available:
#' \code{c("max", "all")}. Default is "max", which outputs the matrix containing
#' only the maximum coefficient. To get the coefficient of all PCs, set this
#' argument as "all". \code{level = "all"} can be used only for one dataset.
#' @param scale Default is \code{FALSE}. If it is set to \code{TRUE}, dataset
#' will be row normalized.
#'
#' @return A data frame containing the maximum pearson correlation coefficient
#' between the top `n` PCs of the dataset and pre-calculated average loadings
#' (in row) of training datasets (\code{score} column). It also contains other
#' metadata associated with each RAV: \code{PC} for one of the top `n` PCs of 
#' the dataset that results in the given \code{score}, \code{sw} for the average
#' silhouette width of the RAV, \code{cl_size} for the size of each RAV.
#'
#' If the input for \code{dataset} argument is a list of different datasets,
#' each row of the output represents a new dataset for test, and each column
#' represents clusters from training datasets. If \code{level = "all"}, a list
#' containing the matrices of the pearson correlation coefficient between all
#' top 8 PCs of the datasets and avgLoading.
#'
#' @examples
#' data(miniRAVmodel)
#' library(bcellViper)
#' data(bcellViper)
#' validate(dset, miniRAVmodel)
#' validate(dset, miniRAVmodel, maxFrom = "avgLoading")
#'
#' @export
validate <- function(dataset, 
                     RAVmodel, 
                     n = NULL,
                     method = "pearson",
                     maxFrom = "PC", 
                     level = "max", 
                     scale = FALSE) {
  
    if (!is.list(dataset)) {
        n <- .set_n(dataset, n)
        if (ncol(dataset) < n) {
            stop("n should be equal or less than the number of samples.")}
    } else {
        if (is.null(n)) {n <- 8}
        if (any(lapply(dataset, ncol) < n)) {
            stop("n should be equal or less than the number of samples.")}
        if (level == "all") {
            stop("'level = \"all\"' is not available for a list of datasets.")}
    }

    sw <- silhouetteWidth(RAVmodel)
    cl_size <- S4Vectors::metadata(RAVmodel)$size
    avgLoading <- SummarizedExperiment::assay(RAVmodel)

    # The maximum correlation coefficient among PCs
    if (maxFrom == "PC") {
        # For a single dataset
        if (!is.list(dataset)) {
            x <- .loadingCor(dataset, avgLoading, n, method, scale)
            if (level == "max") {
                z <- apply(x, 1, max) %>% as.data.frame   # rowMax
                z$PC <- apply(x, 1, which.max)
                colnames(z)[1] <- "score"
                z$sw <- sw   # Silhouette width
                z$cl_size <- cl_size   # Cluster size
                z$cl_num <- readr::parse_number(rownames(z))   # Cluster number
                res <- z
                return(res)
            } else if (level == "all") {
                res <- x
                return(res)
            }
        } else {
            # For a list of datasets
            x <- lapply(dataset, .loadingCor, avgLoading, n, method, scale)
            l <- nrow(x[[1]]) # the number of RAVs in validation output
            if (level == "max") {
                z <- vapply(x, function(y) {apply(y, 1, max)},
                            FUN.VALUE = numeric(l))
                zPC <- vapply(x, function(y) {apply(y, 1, which.max)},
                              FUN.VALUE = integer(l))
                colnames(zPC) <- paste0(colnames(zPC), "_PC")
                res <- cbind(z, zPC)
                return(res)
            } else if (level == "all") {
                res <- x
                return(res)
            }
        }
    }

    # The maximum correlation coefficient among avgLoadings
    if (maxFrom == "avgLoading") {
        if (!is.list(dataset)) {
            x <- .loadingCor(dataset, avgLoading, n, method)
            if (level == "max") {
                z <- apply(x, 2, max) %>% as.data.frame # colMax
                max_z_ind <- apply(x, 2, which.max)
                z$validated_RAV <- rownames(x)[max_z_ind]
                colnames(z)[1] <- "score"
                return(z)
            } else if (level == "all") {
                return(x)
            }
        } else {
            x <- lapply(dataset, .loadingCor, avgLoading, method)
            if (level == "max") {
                z <- apply(x, 2, max)
                return(z)
            } else if (level == "all") {
                return(x)
            }
        }
    }
}

context("Test validate function handles differnet types of inputs")

data(miniRAVmodel)
data(miniTCGA)
library(bcellViper)
data(bcellViper)
microTCGA <- readRDS("microTCGA.rds")


test_that("validate works with a single dataset", {
  val_all <- validate(dset, miniRAVmodel)
  val_se <- validate(miniTCGA[[1]], miniRAVmodel)

  expect_true(is.data.frame(val_all))
  expect_true(is.data.frame(val_se))
})


test_that("validate works with a list of input datasets", {
  val_miniTCGA <- validate(miniTCGA, miniRAVmodel)

  expect_true(is.matrix(val_miniTCGA))
  expect_equal(dim(val_miniTCGA), c(20, 4))
})


test_that("Filter out invalid inputs for validate function", {
  expect_error(validate(microTCGA, miniRAVmodel),
               "n should be equal or less than the number of samples.")
  expect_error(validate(miniTCGA, miniRAVmodel, level = "all"),
               "'level = \"all\"' is not available for a list of datasets.")
})


test_that("different arguments", {
  res <- validate(miniTCGA[[1]], miniRAVmodel, method = "spearman",
                  maxFrom = "avgLoading", level = "all")
  
  expect_equal(dim(res), c(20, 8))
  expect_equal(round(res[1,1], 4), 0.1949)
})

#' Search the top enriched pathways for RAV
#'
#' @param RAVmodel PCAGenomicSignatures object.
#' @param ind An integer for RAV you want to check the enriched pathways.
#' @param n A number of top enriched pathways to output. Default is 5.
#' @param abs Default is \code{FALSE}. If it's set to \code{TRUE}, the enriched
#' pathways will be listed based on \code{abs(NES)}.
#'
#' @return A data frame with \code{n} rows and 4 columns;
#' Description, NES, pvalue, and qvalues
#'
#' @examples
#' data(miniRAVmodel)
#' annotateRAV(miniRAVmodel, ind = 695)
#'
#' @export
annotateRAV <- function(RAVmodel, ind, n = 5, abs = FALSE) {

  # extract GSEA results of the given cluster
  cl_name <- paste0("RAV", ind)
  annotatedCluster <- gsea(RAVmodel)[[cl_name]]

  # absolute value of NES
  FUN <- function(x) {if (abs) {abs(x)} else {I(x)}}

  # subset GSEA results
  topAnnotation <- annotatedCluster[
    order(FUN(annotatedCluster$NES), decreasing = TRUE),,drop = FALSE]
  topAnnotation <- topAnnotation[seq_len(n),]
  rownames(topAnnotation) <- NULL

  return(topAnnotation)
}

context("Test annotateRAV function")

data(miniRAVmodel)

test_that("Confirm the output structure", {
  res <- annotateRAV(miniRAVmodel, ind = 695)
  des <- res$Description
  ## RAVmodel version before 1.1.1
  # val <- c("IRIS_Bcell-Memory_IgG_IgA", "DMAP_BCELLA3",
  #          "IRIS_Bcell-Memory_IgM", "IRIS_Bcell-naive", "DMAP_BCELLA4")
  ## RAVmodel version above 1.1.1
  val <- c("IRIS_Bcell-Memory_IgG_IgA", "DMAP_BCELLA3",
           "IRIS_Bcell-naive", "IRIS_Bcell-Memory_IgM", "DMAP_BCELLA2")

  expect_equal(dim(res), c(5, 4))
  expect_equal(colnames(res), c("Description","NES", "pvalue", "qvalues"))
  expect_equal(des, val)
})

test_that("Test abs argument", {
  res2 <- annotateRAV(miniRAVmodel, ind = 1076)
  res3 <- annotateRAV(miniRAVmodel, ind = 1076, abs = TRUE)
  expect_equal(res2[1,1], "REACTOME_METABOLISM_OF_PROTEINS")
  expect_equal(res3[1,1], "REACTOME_CELL_CYCLE")
})

#' List the available RAVmodels
#' 
#' @param simplify Default is \code{TRUE}. If it is set to \code{FALSE}, the
#' additional metadata of different versions of RAVmodel
#' @return Under the default, this function will return a data frame with four
#' columns - prior, version, update, pkg_version. 
#' \itemize{
#'     \item \code{prior} : Different gene sets used for RAVmodel annotation.
#'     Currently, two are available - \code{C2} for MSigDB C2 (curated gene 
#'     sets), and \code{PLIERpriors} for bloodCellMarkersIRISDMAP, svmMarkers, 
#'     and canonicalPathways
#'     \item \code{version} : RAVmodel's version, which can be an input for 
#'     \code{version} argument of \code{\link{getModel}} function
#'     \item \code{update} : Date the RAVmodel is updated
#'     \item \code{pkg_version} : Compatible version of GenomicSuperSignature
#' }
#'
#' @examples 
#' availableRAVmodel()
#' 
#' @export
availableRAVmodel <- function(simplify = TRUE) {
    dir <- system.file("extdata", package = "GenomicSuperSignature")
    map <- utils::read.table(file.path(dir, "availableRAVmodel.csv"),
                             sep = ",", header = TRUE)

    if (isTRUE(simplify)) {
        available_ind <- which(map$gcp == "TRUE")
        map <- map[available_ind, 
                   c("prior", "version", "update", "pkg_version")]
    }
    
    return(map)
}

---
title: "Structure and content of RAVmodel"
author: "Sehyun Oh"
date: "`r format(Sys.time(), '%B %d, %Y')`"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Introduction on RAVmodel}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    number_sections: yes
    toc: yes
    toc_depth: 4
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE, comment = "#>" 
)
```

# Citing GenomicSuperSignature

Please cite GenomicSuperSignature as follows:

Oh, S., Geistlinger, L., Ramos, M. _et al._ GenomicSuperSignature facilitates interpretation of RNA-seq experiments through robust, efficient comparison to public databases. Nat Commun 13, 3695 (2022). https://doi.org/10.1038/s41467-022-31411-3

# Setup

## Install and load package
```{r eval = FALSE}
if (!require("BiocManager"))
    install.packages("BiocManager")
BiocManager::install("GenomicSuperSignature")
```

```{r results="hide", message=FALSE, warning=FALSE}
library(GenomicSuperSignature)
```


## Download RAVmodel
You can download GenomicSuperSignature from Google Cloud bucket using
`GenomicSuperSignature::getModel` function. Currently available models are 
built from top 20 PCs of 536 studies (containing 44,890 samples) containing 
13,934 common genes from each of 536 study's top 90% varying genes based on 
their study-level standard deviation. There are two versions of this RAVmodel
annotated with different gene sets for GSEA: MSigDB C2 (`C2`) and three 
priors from PLIER package (`PLIERpriors`). In this vignette, we are showing 
the `C2` annotated model.

Note that the first interactive run of this code, you will be asked to allow
R to create a cache directory. The model file will be stored there and 
subsequent calls to `getModel` will read from the cache.

```{r load_model}
RAVmodel <- getModel("C2", load=TRUE)
```


# Content of RAVmodel
`RAVindex` is a matrix containing genes in rows and RAVs in columns. `colData` 
slot provides the information on each RAVs, such as GSEA annotation and 
studies involved in each cluster. `metadata` slot stores model construction 
information. `trainingData` slot contains the information on individual 
studies in training dataset, such as MeSH terms assigned to each study.

```{r}
RAVmodel

version(RAVmodel)
geneSets(RAVmodel)
```

<center>
<img src="https://raw.githubusercontent.com/shbrief/GenomicSuperSignaturePaper/master/inst/images/GSig_RAVmodel.png" width="90%" height="90%"/>
</center>

## RAVindex
*R*eplicable *A*xis of *V*ariation (RAV) index is the main component of
GenomicSuperSignature. It serves as an index connecting new datasets and the 
existing database. You can access it through `GenomicSuperSignature::RAVindex` 
(equivalent of `SummarizedExperiment::assay`). Rows are genes and columns are 
RAVs.

Here, RAVmodel consists of 13,934 genes and 4,764 RAVs.

```{r}
class(RAVindex(RAVmodel))
dim(RAVindex(RAVmodel))
RAVindex(RAVmodel)[1:4, 1:4]
```


## Metadata for RAVmodel
Metadata slot of RAVmodel contains information related to the model building.

```{r}
names(metadata(RAVmodel))
```

* `cluster` : cluster membership of each PCs from the training dataset      
* `size` : an integer vector with the length of clusters, containing the number
of PCs in each cluster   
* `k` : the number of all clusters in the given RAVmodel   
* `n` : the number of top PCs kept from each study in the training dataset
* `geneSets` : the name of gene sets used for GSEA annotation   
* `MeSH_freq` : the frequency of MeSH terms associated with the training 
dataset. MeSH terms like 'Humans' and 'RNA-seq' are top ranked (which is very 
expected) because the training dataset of this model is Human RNA sequencing
data. 
* `updateNote` : a brief note on the given model's specification
* `version` : the version of the given model

```{r}
head(metadata(RAVmodel)$cluster)
head(metadata(RAVmodel)$size)
metadata(RAVmodel)$k
metadata(RAVmodel)$n
geneSets(RAVmodel)
head(metadata(RAVmodel)$MeSH_freq)
updateNote(RAVmodel)  
metadata(RAVmodel)$version
```


## Studies in each RAV
You can find which studies are in each cluster using `studies` method. Output is
a list with the length of clusters, where each element is a character vector 
containing the name of studies in each cluster.

```{r}
length(studies(RAVmodel))
studies(RAVmodel)[1:3]
```

You can check which PC from different studies are in RAVs using `PCinRAV`.

```{r}
PCinRAV(RAVmodel, 2)
```


## Silhouette width for each RAV
Silhouette width ranges from -1 to 1 for each cluster. Typically, it is 
interpreted as follows:   
  - Values close to 1 suggest that the observation is well matched to the 
  assigned cluster   
  - Values close to 0 suggest that the observation is borderline matched 
  between two clusters   
  - Values close to -1 suggest that the observations may be assigned to the 
  wrong cluster   
  
For RAVmodel, the average silhouette width of each cluster is a quality control
measure and suggested as a secondary reference to choose proper RAVs, 
following validation score.

```{r}
x <- silhouetteWidth(RAVmodel)
head(x)   # average silhouette width of the first 6 RAVs
```


## GSEA on each RAV
Pre-processed GSEA results on each RAV are stored in RAVmodel and can be 
accessed through `gsea` function.

```{r}
class(gsea(RAVmodel))
class(gsea(RAVmodel)[[1]])
length(gsea(RAVmodel))
gsea(RAVmodel)[1]
```


## MeSH terms for each study
You can find MeSH terms associated with each study using `mesh` method. 
Output is a list with the length of studies used for training. Each element of
this output list is a data frame containing the assigned MeSH terms and the 
detail of them. The last column `bagOfWords` is the frequency of the MeSH term 
in the whole training dataset.

```{r}
length(mesh(RAVmodel))
mesh(RAVmodel)[1]
```


## PCA summary for each study
PCA summary of each study can be accessed through `PCAsummary` method. Output 
is a list with the length of studies, where each element is a matrix containing 
PCA summary results: standard deviation (SD), variance explained by each PC 
(Variance), and the cumulative variance explained (Cumulative).

```{r}
length(PCAsummary(RAVmodel))
PCAsummary(RAVmodel)[1]
```


# Other relevant code

The workflow to build the RAVmodel is available from https://github.com/shbrief/model_building which is archived in Zenodo with the identifier https://doi.org/10.5281/zenodo.6496552. All analyses presented in the GenomicSuperSignatures manuscript are reproducible using code accessible from https://github.com/shbrief/GenomicSuperSignaturePaper/ and archived in Zenodo with the identifier [https://doi.org/10.5281/zenodo.6496612].


# Session Info

<details>
```{r}
sessionInfo()
```
</details>

---
title: "Analysis by GenomicSuperSignature"
date: "`r Sys.Date()`"
output:
  BiocStyle::html_document:
    toc: true
    toc_float: false
    toc_depth: 3
params: 
    val_all: val_all
    dat: dat
    RAVmodel: RAVmodel
    inputName: inputName
    numOut: numOut
    score: score
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE)
```

# RAVs best represents your dataset
The *validation* provides a quantitative representation of the relevance
between your dataset and RAVs. Below shows the top 6 validated RAVs and 
the complete result is saved as `{input_name}_validate.csv`. 

```{r}
head(params$val_all)
```

## Heatmap Table
`heatmapTable` takes validation results as its input and displays them into 
a two panel table: the top panel shows the average silhouette width (avg.sw) 
and the bottom panel displays the validation score.

`heatmapTable` can display different subsets of the validation output. For 
example, if you specify `scoreCutoff`, any validation result above that score 
will be shown. If you specify the number (n) of top validation results through 
`num.out`, the output will be a n-columned heatmap table. You can also use the 
average silhouette width (`swCutoff`), the size of cluster (`clsizecutoff`), 
one of the top 8 PCs from the dataset (`whichPC`).

Here, we print out top `r params$numOut` validated RAVs with average silhouette 
width above 0.

```{r out.height="45%", out.width="45%", message=FALSE, warning=FALSE}
heatmapTable(params$val_all, num.out = params$numOut, swCutoff = 0)
```

## Sample Score
RAV-assigned scores for each sample can be used to compare the features 
represented by the given RAV across different datasets. Below shows a part 
of sample scores in a heatmap, where scores are assigned to each sample (row) 
from each RAV (column).
The complete result is saved as `{input_name}__sampleScore.csv`.

```{r out.height="65%", out.width="65%"}
k <- min(nrow(params$score), 10)
sampleScoreHeatmap(params$score[seq_len(k),], 
                   dataName = params$inputName, 
                   modelName = "RAV",
                   column_names_gp = 10)
```

## Interactive Graph
Under the default condition, `plotValidate` plots validation results of all non 
single-element RAVs in one graph, where x-axis represents average silhouette 
width of the RAVs (a quality control measure of RAVs) and y-axis represents 
validation score. We recommend users to focus on RAVs with higher validation 
score and use average silhouette width as a secondary criteria. 

```{r out.height="80%", out.width="80%"}
plotValidate(params$val_all, interactive = TRUE)
```

Note that `interactive = TRUE` will result in a zoomable, interactive plot that
included tooltips, which is saved as `{input_name}_validate_plot.html` file. 

You can hover each data point for more information:    

- **sw** : the average silhouette width of the cluster   
- **score** : the top validation score between 8 PCs of the dataset and RAVs   
- **cl_size** : the size of RAVs, represented by the dot size   
- **cl_num** : the RAV number. You need this index to find more information 
about the RAV.      
- **PC** : test dataset's PC number that validates the given RAV. Because we 
used top 8 PCs of the test dataset for validation, there are 8 categories. 

If you double-click the PC legend on the right, you will enter an 
individual display mode where you can add an additional group of data 
point by single-click. 


# Prior information associated to your dataset
```{r echo=FALSE}
validated_ind <- validatedSignatures(params$val_all, num.out = params$numOut,
                                     swCutoff = 0, indexOnly = TRUE)

# In case, there are fewer validated_ind than the number of outputs user set
n <- min(params$numOut, length(validated_ind), na.rm = TRUE)
```

## MeSH terms in wordcloud
```{r out.height="60%", out.width="60%", fig.width=8, fig.height=8}
for (i in seq_len(n)) {
    set.seed(1)
    print(paste0("MeSH terms related to RAV", validated_ind[i]))
    drawWordcloud(params$RAVmodel, validated_ind[i])
}
```

## GSEA
The complete result is saved as `{input_name}_genesets_RAV*.csv`. 
```{r}
res_all <- vector(mode = "list", length = n)
for (i in seq_len(n)) {
    RAVnum <- validated_ind[i]
    RAVname <- paste0("RAV", RAVnum)
    res <- gsea(params$RAVmodel)[[RAVname]]
    res_all[[i]] <- head(res)
    names(res_all)[i] <- paste0("Enriched gene sets for RAV", validated_ind[i])
}
res_all
```

## Publication
The complete result is saved as `{input_name}_literatures_RAV*.csv`. 
```{r}
res_all <- vector(mode = "list", length = n)
for (i in seq_len(n)) {
    RAVnum <- validated_ind[i]
    res <- findStudiesInCluster(params$RAVmodel, RAVnum, studyTitle = TRUE)
    res_all[[i]] <- head(res)
    names(res_all)[i] <- paste0("Studies related to RAV", validated_ind[i])
}
res_all
```

---
title: "GenomicSuperSignature - Quickstart"
author: "Sehyun Oh"
date: "`r format(Sys.time(), '%B %d, %Y')`"
abstract: |
    This vigenette demostrates a basic usage of GenomicSuperSignature. More
    extensive and biology-relavant use cases are available [**HERE**](https://shbrief.github.io/GenomicSuperSignaturePaper/).
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Quickstart}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    number_sections: yes
    toc: yes
    toc_depth: 4
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  comment = "#>", collapse = TRUE, message = FALSE, warning = FALSE,
  fig.align='center'
)
```


# Setup
## Install and load package
```{r eval = FALSE}
if (!require("BiocManager"))
    install.packages("BiocManager")
BiocManager::install("GenomicSuperSignature")
BiocManager::install("bcellViper")
```

```{r results="hide", message=FALSE, warning=FALSE}
library(GenomicSuperSignature)
library(bcellViper)
```


## Download RAVmodel
You can download GenomicSuperSignature from Google Cloud bucket using
`GenomicSuperSignature::getModel` function. Currently available models are 
built from top 20 PCs of 536 studies (containing 44,890 samples) containing 
13,934 common genes from each of 536 study's top 90% varying genes based on 
their study-level standard deviation. There are two versions of this RAVmodel
annotated with different gene sets for GSEA - MSigDB C2 (`C2`) and three 
priors from PLIER package (`PLIERpriors`). 

The demo in this vignette is based on human B-cell expression data, so we are 
using the `PLIERpriors` model annotated with blood-associated gene sets. 

Note that the first interactive run of this code, you will be asked to allow
R to create a cache directory. The model file will be stored there and 
subsequent calls to `getModel` will read from the cache.

```{r load_model}
RAVmodel <- getModel("PLIERpriors", load=TRUE)
RAVmodel

version(RAVmodel)
```


## Example dataset
The human B-cell dataset (Gene Expression Omnibus series GSE2350) consists of 
211 normal and tumor human B-cell phenotypes. This dataset was generated on 
Affymatrix HG-U95Av2 arrays and stored in an ExpressionSet object with 6,249 
features x 211 samples.

```{r message=FALSE, warning=FALSE}
data(bcellViper)
dset
```

You can provide your own expression dataset in any of these formats: simple 
matrix, ExpressionSet, or SummarizedExperiment. Just make sure that genes are 
in a 'symbol' format. 




# Which RAV best represents the dataset?
`validate` function calculates validation score, which provides a quantitative 
representation of the relevance between a new dataset and RAV. RAVs that give 
the validation score is called _*validated RAV*_. The validation results can 
be displayed in different ways for more intuitive interpretation. 

```{r}
val_all <- validate(dset, RAVmodel)
head(val_all)
```


## HeatmapTable
`heatmapTable` takes validation results as its input and displays them into 
a two panel table: the top panel shows the average silhouette width (avg.sw) 
and the bottom panel displays the validation score.

`heatmapTable` can display different subsets of the validation output. For 
example, if you specify `scoreCutoff`, any validation result above that score 
will be shown. If you specify the number (n) of top validation results through 
`num.out`, the output will be a n-columned heatmap table. You can also use the 
average silhouette width (`swCutoff`), the size of cluster (`clsizecutoff`), 
one of the top 8 PCs from the dataset (`whichPC`).

Here, we print out top 5 validated RAVs with average silhouette width above 0.

```{r out.height="80%", out.width="80%", message=FALSE, warning=FALSE}
heatmapTable(val_all, RAVmodel, num.out = 5, swCutoff = 0)
```


## Interactive Graph
Under the default condition, `plotValidate` plots validation results of all non 
single-element RAVs in one graph, where x-axis represents average silhouette 
width of the RAVs (a quality control measure of RAVs) and y-axis represents 
validation score. We recommend users to focus on RAVs with higher validation 
score and use average silhouette width as a secondary criteria. 

```{r out.height="75%", out.width="75%", plotValidate_function}
plotValidate(val_all, interactive = FALSE)
```

Note that `interactive = TRUE` will result in a zoomable, interactive plot
that included tooltips. 

You can hover each data point for more information:    

- **sw** : the average silhouette width of the cluster   
- **score** : the top validation score between 8 PCs of the dataset and RAVs   
- **cl_size** : the size of RAVs, represented by the dot size   
- **cl_num** : the RAV number. You need this index to find more information 
about the RAV.      
- **PC** : test dataset's PC number that validates the given RAV. Because we 
used top 8 PCs of the test dataset for validation, there are 8 categories. 

If you double-click the PC legend on the right, you will enter an 
individual display mode where you can add an additional group of data 
point by single-click. 


# What kinds of information can you access through RAV?
GenomicSuperSignature connects different public databases and prior information 
through RAVmodel, creating the knowledge graph illustrated below. Through RAVs, 
you can access and explore the knowledge graph from multiple entry points such 
as gene expression profiles, publications, study metadata, keywords in MeSH 
terms and gene sets.

</br>

<center>
<img src="https://raw.githubusercontent.com/shbrief/GenomicSuperSignaturePaper/master/inst/images/GSig_knowledge_graph.png" width="120%" height="120%"/>
</center>


## MeSH terms in wordcloud
You can draw a wordcloud with the enriched MeSH terms of RAVs. Based on the 
heatmap table above, three RAVs (2538, 1139, and 884) show the high validation 
scores with the positive average silhouette widths, so we draw wordclouds of 
those RAVs using `drawWordcloud` function. You need to provide RAVmodel and 
the index of the RAV you are interested in.

Index of validated RAVs can be easily collected using `validatedSingatures`
function, which outputs the validated index based on `num.out`, PC from dataset 
(`whichPC`) or any `*Cutoff` arguments in the same way as `heatmapTable`. Here, 
we choose the top 3 RAVs with the average silhouette width above 0, which will 
returns RAV2538, RAV1139, and RAV884 as we discussed above.

```{r}
validated_ind <- validatedSignatures(val_all, RAVmodel, num.out = 3, 
                                     swCutoff = 0, indexOnly = TRUE)
validated_ind
```

And we plot the wordcloud of those three RAVs.

```{r out.height="60%", out.width="60%"}
set.seed(1) # only if you want to reproduce identical display of the same words
drawWordcloud(RAVmodel, validated_ind[1])
drawWordcloud(RAVmodel, validated_ind[2])
drawWordcloud(RAVmodel, validated_ind[3])
```


## GSEA
### Associated gene sets of validated RAV
You can directly access the GSEA outputs for each RAV using `annotateRAV` 
function. Based on the wordclouds, RAV1139 seems to be associated with B-cell.

```{r}
annotateRAV(RAVmodel, validated_ind[2]) # RAV1139
```

If you want to check the enriched pathways for multiple RAVs, use
`subsetEnrichedPathways` function instead. 
```{r}
subsetEnrichedPathways(RAVmodel, validated_ind[2], include_nes = TRUE)

subsetEnrichedPathways(RAVmodel, validated_ind, include_nes = TRUE)
```


### Search enriched pathways through keyword
You can also find the RAVs annotated with the keyword-containing pathways using
`findSignature` function. Without the `k` argument, this function outputs a 
data frame with two columns: the number of RAVs (`Freq` column) with the 
different numbers of keyword-containing, enriched pathways 
(`# of keyword-containing pathways` column).

Here, we used the keyword, "Bcell".
```{r}
findSignature(RAVmodel, "Bcell")
```

There are two RAVs with five keyword-containing pathways (row 6). We can check
which RAVs they are.
```{r}
findSignature(RAVmodel, "Bcell", k = 5)
```

Enriched pathways are ordered by NES and you can check the rank of any keyword-
containing pathways using `findKeywordInRAV`.
```{r}
findKeywordInRAV(RAVmodel, "Bcell", ind = 695)
```

You can check all enriched pathways of RAV using `subsetEnrichedPathways` 
function. If `both=TRUE`, both the top and bottom enriched pathways will 
be printed.
```{r}
## Chosen based on validation/MeSH terms
subsetEnrichedPathways(RAVmodel, ind = validated_ind[2], n = 3, both = TRUE)

## Chosen based on enriched pathways
subsetEnrichedPathways(RAVmodel, ind = 695, n = 3, both = TRUE)
subsetEnrichedPathways(RAVmodel, ind = 953, n = 3, both = TRUE)
subsetEnrichedPathways(RAVmodel, ind = 1994, n = 3, both = TRUE)
```

## Related prior studies
You can find the prior studies related to a given RAV using 
`findStudiesInCluster` function.

```{r}
findStudiesInCluster(RAVmodel, validated_ind[2])
```

You can easily extract the study name with the `studyTitle=TRUE` parameter.
```{r}
findStudiesInCluster(RAVmodel, validated_ind[2], studyTitle = TRUE)
```



# Session Info

<details>
```{r}
sessionInfo()
```
</details>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findSignature.R
\name{findSignature}
\alias{findSignature}
\title{Find the RAVs with the keyword-containing enriched pathways}
\usage{
findSignature(RAVmodel, keyword, n = 5, k = NULL)
}
\arguments{
\item{RAVmodel}{PCAGenomicSignatures-object}

\item{keyword}{A character vector. If you are searching for multiple keywords
at the same time, use \code{\link{paste}} with \code{collapse="|"} argument.}

\item{n}{The number of top ranked (based on abs(NES)) pathways you want to
search your keyword}

\item{k}{The number of keyword-containing pathways you want to get the RAV
number. Under default (\code{NULL}), the output will be a data frame with two
columns: '# of keyword-containing pathways' and 'Freq'. If you assign the
value for this argument, the output will be an integer vector containing the
RAV index.}
}
\value{
A data frame or integer vector depending on the parameter \code{k}.
}
\description{
This function finds RAVs containing the keyword you provide. If you provide
"the number of keyword-containing pathways per RAV" in argument \code{k},
it will give you the RAV number.
}
\examples{
data(miniRAVmodel)
findSignature(miniRAVmodel, "Bcell")
findSignature(miniRAVmodel, "Bcell", k = 5)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmapTable.R
\name{heatmapTable}
\alias{heatmapTable}
\title{Validation result in heatmap format}
\usage{
heatmapTable(
  val_all,
  RAVmodel,
  ind = NULL,
  num.out = 5,
  scoreCutoff = NULL,
  swCutoff = NULL,
  clsizeCutoff = NULL,
  breaks = c(0, 0.5, 1),
  colors = c("white", "white smoke", "red"),
  column_title = NULL,
  row_title = NULL,
  whichPC = NULL,
  filterMessage = TRUE,
  ...
)
}
\arguments{
\item{val_all}{An output matrix from \code{\link{validate}} function with the
parameter \code{level = "max"}. Subset of this matrix is plotted as a heatmap
using \code{\link[ComplexHeatmap]{Heatmap}}}

\item{RAVmodel}{PCAGenomicSignatures-class object. RAVmodel used to prepare
\code{val_all} input.}

\item{ind}{An integer vector. If this parameter is provided, the other
parameters, \code{num.out, scoreCutoff, swCutoff, clsizeCutoff} will be
ignored and the heatmap table containing only the provided index will be
printed.}

\item{num.out}{A number of highly validated RAVs to output. Default is 5.
If any of the cutoff parameters are provided, \code{num.out} or the number of
filtered RAVs, whichever smaller, will be chosen.}

\item{scoreCutoff}{A numeric value for the minimum correlation (not include).
If \code{val_all} input is from multiple studies, the default is 0.7 and this is
the only cutoff criteria considered: \code{swCutoff} and \code{clsizeCutoff}
will be ignored.}

\item{swCutoff}{A numeric value for the minimum average silhouette width.}

\item{clsizeCutoff}{A integer value for the minimum cluster size.}

\item{breaks}{A numeric vector of length 3. Number represents the values
assigned to three colors. Default is \code{c(0, 0.5, 1)}.}

\item{colors}{A character vector of length 3. Each represents the color
assigned to three breaks. Default is \code{c("white", "white smoke", "red")}.}

\item{column_title}{A character string. Provide the column title.}

\item{row_title}{A character string. Provide the row title.}

\item{whichPC}{An integer value between 1 and 8. PC number of your data to
check the validated signatures with. Under the default (\code{NULL}), it
outputs top scored signatures with any PC of your data.}

\item{filterMessage}{A logical. Under the default \code{TRUE}, any output
RAV belong to the filtering list will give a message. Silence this message
with \code{filterMessage=FALSE}. You can check the filter list using
\code{data("filterList")}.}

\item{...}{any additional argument for \code{\link[ComplexHeatmap]{Heatmap}}}
}
\value{
A heatmap displaying the subset of the validation result that met the
given cutoff criteria. If \code{val_all} input is from a single dataset, the
output heatmap will contain both score and average silhouette width for each
cluster.

If \code{val_all} input is from multiple studies, the output heatmap's rows
will represent each study and the columns will be RAVs, which meet
\code{scoreCutoff} for any of the input studies.
}
\description{
This function subsets \code{\link{validate}} outputs with different criteria
and visualize it in a heatmap-like table.
}
\examples{
data(miniRAVmodel)
library(bcellViper)
data(bcellViper)

## Single dataset
val_all <- validate(dset, miniRAVmodel)
heatmapTable(val_all, miniRAVmodel, swCutoff = 0)

## A list of datasets
val_all2 <- validate(miniTCGA, miniRAVmodel)
heatmapTable(val_all2, miniRAVmodel)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/buildAvgLoading.R
\name{buildAvgLoading}
\alias{buildAvgLoading}
\title{Calculate average loadings of each cluster}
\usage{
buildAvgLoading(dat, k, n = 20, cluster = NULL, study = TRUE)
}
\arguments{
\item{dat}{A data frame. Each row represents principle components from
different training datasets. Columns are genes used for PCA analysis.}

\item{k}{The number of clusters used for hierarchical clustering}

\item{n}{The number of top principle components from each datasets used for
model building. Default is 20.}

\item{cluster}{Provide pre-defined cluster membership of your data.}

\item{study}{Under default (\code{TRUE}), studies involved in each cluster
will be added in the output.}
}
\value{
A named list of 6 elements is returned. It contains:
\describe{
   \item{\code{cluster}}{A numeric vector on cluster membership of PCs}
   \item{\code{size}}{A integer vector on the size of clusters}
   \item{\code{avgLoading}}{A matrix of average loadings. Columns for
   clusters and rows for genes}
   \item{\code{k}}{The number of clusters}
   \item{\code{n}}{The number of top PCs used for clustering}
   \item{\code{studies}}{A list of character vector containing studies in
   each cluster}
}
}
\description{
Calculate average loadings of each cluster
}
\examples{
data(miniAllZ)
data(res_hcut)
res <- buildAvgLoading(miniAllZ, k = 40, cluster = res_hcut$cluster)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotatePC.R
\name{annotatePC}
\alias{annotatePC}
\title{Annotate top PCs from the dataset}
\usage{
annotatePC(
  PCnum,
  val_all,
  RAVmodel,
  n = 5,
  scoreCutoff = 0.5,
  nesCutoff = NULL,
  simplify = TRUE,
  abs = FALSE,
  trimed_pathway_len = 45
)
}
\arguments{
\item{PCnum}{A numeric vector. PC number of your dataset to retrieve
annotation results for. The vector can contain any integer number among
\code{1:8}.}

\item{val_all}{The output from \code{\link{validate}}}

\item{RAVmodel}{The RAVmodel used to generate the input for the argument,
\code{val_all}.}

\item{n}{An integer. Default is 5. The number of the top enriched pathways
to print out. If there are fewer than n pathways passed the cutoff, it will
print out \code{NA}.}

\item{scoreCutoff}{A numeric value for the minimum correlation between loadings
of the dataset principal component and the RAV. Default is 0.5.}

\item{nesCutoff}{A numeric value for the minimum Normalized Enrichment Score
(NES) for the enrichment analysis. Default is \code{NULL}
The suggested value is 3.}

\item{simplify}{A logical. Under default (\code{TRUE}), the output will be a
data frame with the number of column same as the length of \code{PCnum}
argument, and the number of row same as the \code{n} argument. If it is set
to \code{FALSE}, the output will be a list with the length of \code{PCnum}
argument, where each element is a data frame containing detailed GSEA output
of enriched pathways.}

\item{abs}{Default is \code{FALSE}. If it's set to \code{TRUE}, the enriched
pathways will be listed based on absolute value of the Normalized Enrichment
Score (NES).}

\item{trimed_pathway_len}{Positive integer values, which is the display width
of pathway names. Default is 45.}
}
\value{
A data frame of a list based on the \code{simplify} argument. Check
the output detail above.
}
\description{
This function finds the RAV with the highest validation score (including
RAVs with negative silhouette width) for specified PC of the dataset and
returns the top enriched pathways.
}
\examples{
data(miniRAVmodel)
library(bcellViper)
data(bcellViper)
val_all <- validate(dset, miniRAVmodel)
annotatePC(2, val_all, miniRAVmodel)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotValidate.R
\name{plotValidate}
\alias{plotValidate}
\title{Plot validation results in an interactive graph}
\usage{
plotValidate(
  val_all,
  minClusterSize = 2,
  swFilter = FALSE,
  minSilhouetteWidth = 0,
  interactive = FALSE,
  minClSize = NULL,
  maxClSize = NULL,
  colorPalette = "Dark2"
)
}
\arguments{
\item{val_all}{Output from \link{validate} function.}

\item{minClusterSize}{The minimum size of clusters to be included in the
plotting. Default value is 2, so any single-element clusters are excluded.}

\item{swFilter}{If \code{swFilter=TRUE}, only RAV above the cutoff, defined
through \code{minSilhouetteWidth} argument will be plotted. Default is
\code{swFilter=FALSE}}

\item{minSilhouetteWidth}{A minimum average silhouette width to be plotted.
Only effective under \code{swFilter=TRUE} condition. Default is 0.}

\item{interactive}{If set to \code{TRUE}, the output will be interactive
plot. Default is \code{FALSE}.}

\item{minClSize}{The minimum number of PCs in the clusters you want.}

\item{maxClSize}{The maximum number of PCs in the clusters you want.}

\item{colorPalette}{Default is \code{Dark2}. For other color options, please
check \code{\link[ggplot2]{scale_color_brewer}}}
}
\value{
a ggplot object
}
\description{
There are three main information on the graph:
\itemize{
    \item x-axis : Pearson correlation coefficient. Higher value means that
    test dataset and RAV is more tightly associated with.
    \item y-axis : Silhouette width representing the quality of RAVs.
    \item size : The number of studies in each RAV. (= cluster size)
    \item color : Test dataset's PC number that validate each RAV. Because we
    used top 8 PCs of the test dataset, there are 8 categories.
}
}
\examples{
data(miniRAVmodel)
library(bcellViper)
data(bcellViper)
val_all <- validate(dset, miniRAVmodel)
plotValidate(val_all)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drawWordcloud.R
\name{meshTable}
\alias{meshTable}
\title{Build a two-column word/frequency table}
\usage{
meshTable(
  RAVmodel,
  ind,
  rm.noise = NULL,
  weighted = TRUE,
  filterMessage = TRUE
)
}
\arguments{
\item{RAVmodel}{A PCAGenomicSignatures object}

\item{ind}{An index of RAV}

\item{rm.noise}{An integer. Under the default (\code{rm.noise=NULL}), if
cluster size (= \code{s}) is smaller than 8, \code{rm.noise = floor(s*0.5)}.
For clusters with >= 8 PCs, \code{rm.noise = 4}. If \code{rm.noise = 0}, all
the MeSH terms in RAV will be used to draw wordcloud.}

\item{weighted}{A logical. If \code{TRUE}, MeSH terms from each study are
weighted based on the variance explained by the principle component of the
study contributing a give RAV. Default is \code{TRUE}.}

\item{filterMessage}{A logical. Under the default \code{TRUE}, any output
RAV belong to the filtering list will give a message. Silence this message
with \code{filterMessage=FALSE}. You can check the filter list using
\code{data("filterList")}.}
}
\value{
A table with two columns, \code{word} and \code{freq}. MeSH terms in
the defined RAV (by \code{ind} argument) is ordered based on their frequency.
}
\description{
Build a two-column word/frequency table
}
\examples{
data(miniRAVmodel)
meshTable(miniRAVmodel,1139)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drawWordcloud.R
\name{drawWordcloud}
\alias{drawWordcloud}
\title{Draw wordcloud using the collection of RAVs' MeSH terms}
\usage{
drawWordcloud(
  RAVmodel,
  ind,
  rm.noise = NULL,
  scale = c(3, 0.5),
  weighted = TRUE,
  drop = NULL,
  filterMessage = TRUE
)
}
\arguments{
\item{RAVmodel}{PCAGenomicSignatures object}

\item{ind}{An index of the RAV you want to draw wordcloud.}

\item{rm.noise}{An integer. Under the default (\code{rm.noise=NULL}), if
cluster size (= \code{s}) is smaller than 8, \code{rm.noise = floor(s*0.5)}.
For clusters with >= 8 PCs, \code{rm.noise = 4}. If \code{rm.noise = 0}, all
the MeSH terms in RAV will be used to draw wordcloud.}

\item{scale}{A \code{scale} argument for \code{\link[wordcloud]{wordcloud}}
function}

\item{weighted}{A logical. If \code{TRUE} (default), MeSH terms from each
study are weighted based on the variance explained by the principle component
of the study contributing to a given RAV.}

\item{drop}{A character vector containing MeSH terms to be excluded from word
cloud. Under the default (\code{NULL}), manually selected non-informative
MeSH terms are excluded, which can be viewed through \code{data(droplist)}.}

\item{filterMessage}{A logical. Under the default \code{TRUE}, any output
RAV belong to the filtering list will give a message. Silence this message
with \code{filterMessage=FALSE}. You can check the filter list using
\code{data("filterList")}.}
}
\value{
A word cloud with the MeSH terms associated with the given cluster.
}
\description{
Plot a word cloud using the remaining MeSH terms in the selected
RAV after user-defined filtering.
}
\examples{
data(miniRAVmodel)
drawWordcloud(miniRAVmodel, 1139)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{miniTCGA}
\alias{miniTCGA}
\title{Subset of TCGA-COAD and TCGA-BRCA RNA sequencing datasets}
\format{
A list containing two SummarizedExperiment objects.
}
\usage{
miniTCGA
}
\description{
TCGA-COAD and TCGA-BRCA RNA sequencing data were acquired using
\code{GSEABenchmarkeR::loadEData} and log-transformed. Conversion from
EntrezID to gene symbol was done with \code{EnrichmentBrowser::idMap}. Only
8 samples from each dataset are kept.
}
\author{
Sehyun Oh \email{shbrief@gmail.com}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{droplist}
\alias{droplist}
\title{MeSH terms to be excluded in drawWordcloud function}
\format{
A character vector containing MeSH terms to be excluded.
}
\usage{
droplist
}
\description{
MeSH terms to be excluded in drawWordcloud function
}
\author{
Sehyun Oh \email{shbrief@gmail.com}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findSignature.R
\name{findKeywordInRAV}
\alias{findKeywordInRAV}
\title{Find the rank of your keyword in the RAV's GSEA annotation}
\usage{
findKeywordInRAV(RAVmodel, keyword, ind, n = NULL, includeTotal = TRUE)
}
\arguments{
\item{RAVmodel}{PCAGenomicSignatures-object.}

\item{keyword}{A character vector. If you are searching for multiple keywords
at the same time, use \code{\link{paste}} with \code{collapse="|"} argument.}

\item{ind}{An integer. The RAV number you want to check.}

\item{n}{An integer. The number of top enriched pathways (based on abs(NES))
to search. Under default (\code{NULL}), all the enriched pathways are used.}

\item{includeTotal}{Under the default condition (\code{TRUE}), the total
number of enriched pathways will be also printed out as a part of the output.}
}
\value{
A character containing the rank of keyword-containing pathways
(separated by |), followed by the total number of enriched pathways in
parenthesis.
}
\description{
Once you provide RAVmodel, keyword you're searching for, and the RAV number
to this function, it will give you the abs(NES)-based rank of your keyword in
the enriched pathways of the target RAV. If can be useful to find out how
uniquely your keyword-containing pathways are represented.
}
\examples{
data(miniRAVmodel)
findKeywordInRAV(miniRAVmodel, "Bcell", ind = 695)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findStudiesInCluster.R
\name{findStudiesInCluster}
\alias{findStudiesInCluster}
\title{Find the studies contributing each RAV}
\usage{
findStudiesInCluster(RAVmodel, ind = NULL, studyTitle = FALSE)
}
\arguments{
\item{RAVmodel}{PCAGenomicSignatures object.}

\item{ind}{A numeric vector containing the RAV indexes. Under the default
(\code{NULL}), studies associated with all the RAV indexes will be returned
as a list.}

\item{studyTitle}{Default is \code{FALSE}. This parameter is effective only when
the \code{index} value is specificed. If it's \code{TRUE}, the output will be
a data frame with the study}
}
\value{
A list of character vectors. Under the default condition
(\code{ind = NULL}), all the RAVs will be checked for their contributing
studies and the length of the list will be same as the number of RAVs
(= \code{metadata(x)$k}). If you provide the \code{ind} argument, studies
associated with only the specified RAVs will be returned.
}
\description{
Find the studies contributing each RAV
}
\note{
Mainly used for model building, within \link{buildAvgLoading}.
}
\examples{
data(miniRAVmodel)
findStudiesInCluster(miniRAVmodel, 1076)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{miniAllZ}
\alias{miniAllZ}
\title{Subset of allZ matrix constructed from 8 CRC training datasets}
\format{
A matrix with 100 genes and 160 PCs from 8 training datasets.
}
\source{
https://github.com/shbrief/model_building/tree/main/RAVmodel_8CRC
}
\usage{
miniAllZ
}
\description{
Eight colorectal cancer microarray datasets were used to build RAVmodel and
the intermediate file containing genes and top PCs from each dataset is named
as \code{allZ}. Hierarchical clustering result of \code{allZ} is saved as
\code{res_hcut}. For demonstration, we subset the \code{allZ} matrix with the
first 100 genes, which is named as \code{miniAllZ}.
}
\author{
Sehyun Oh \email{shbrief@gmail.com}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PCAGenomicSignatures-methods.R
\name{PCAGenomicSignatures}
\alias{PCAGenomicSignatures}
\title{Construct \code{PCAGenomicSignatures} object}
\usage{
PCAGenomicSignatures(..., trainingData)
}
\arguments{
\item{...}{Additional arguments for supporting functions.}

\item{trainingData}{A \code{\link[S4Vectors]{DataFrame}} class object for
metadata associated with training data}
}
\value{
PCAGenomicSignatures object with multiple setters or accessors
}
\description{
The default contents of \code{PCAGenomicSignatures} object, with
a set of accessors and setter generic functions, which extract either the
\code{assay}, \code{colData}, \code{metadata}, or \code{trainingData} slots
of a \code{\link{PCAGenomicSignatures-class}} object. When you create this
object, \code{colData$studies} should be populated before adding any
information in \code{trainingData} slot
}
\details{
\itemize{
    \item RAVindex(x) : RAVindex (= avgLoadings) containing genes x RAVs
    \item metadata(x)$cluster : A vector of integers (from 1:k) indicating
    the cluster to which each point is allocated.
    \item metadata(x)$size : The number of PCs in each cluster.
    \item metadata(x)$k : The number of RAVs.
    \item metadata(x)$n : The number of top PCs from each dataset.
    \item metadata(x)$geneSets : Name of the prior gene sets used to annotate
    average loadings.
    \item colData(x)$studies : A list of character vectors containing studies
    contributing to each PC cluster.
    \item colData(x)$silhouetteWidth : A numeric array of average silhouette
    widths of each clusters
    \item colData(x)$gsea : A list of data frames. Each element is a subset
    of outputs from \code{clusterProfiler::GSEA} function.
}
}
\section{Slots}{

\describe{
\item{\code{trainingData}}{A \code{\link[S4Vectors]{DataFrame}} class object for
metadata associated with training data}
}}

\section{Setters}{

Setter method values (i.e., \code{function(x) <- value}):
\itemize{
    \item geneSets<- : A character vector containing the name of gene sets
    used to annotate average loadings
    \item studies<- : A list of character vectors containing gene sets used
    to annotate average loadings
    \item gsea<- : A list of data frames. Each element is a subset of output
    from \code{gseaResult} objects.
    \item metadata<- : A \code{list} object of metadata
    \item `$<-` : A vector to replace the indicated column in \code{colData}
}
}

\section{Accessors}{

All the accessors inherited from \code{SummarizedExperiment} are available
and the additional accessors for \code{PCAGenomicSignatures} specific data
are listed below.
\itemize{
   \item RAVindex : Equivalent to the \code{assay(x)}
   \item geneSets : Access the \code{metadata(x)$geneSets} slot
   \item studies : Access the \code{colData(x)$studies} slot
   \item gsea : Access the \code{colData(x)$gsea}
   \item `$` : Access a column in \code{colData}
   \item trainingData : Access the \code{trainingData} slot
   \item mesh : Access the \code{trainingData(x)$MeSH} slot
   \item PCAsummary : Access the \code{trainingData(x)$PCAsummary} slot
}
}

\examples{
data(miniRAVmodel)
miniRAVmodel

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate.R
\name{validate}
\alias{validate}
\title{Validate new datasets}
\usage{
validate(
  dataset,
  RAVmodel,
  n = NULL,
  method = "pearson",
  maxFrom = "PC",
  level = "max",
  scale = FALSE
)
}
\arguments{
\item{dataset}{Single or a named list of SummarizedExperiment
(RangedSummarizedExperiment, ExpressionSet or matrix) object(s). Columns 
should contain samples (or cells for single-cell data), and the row names
should be in 'gene symbol' format.}

\item{RAVmodel}{PCAGenomicSignatures object.}

\item{n}{A integer. The number of PCs to use for validation. It should be
equal or less then the number of samples in the input dataset. If there are 
>= 8 samples in the input dataset or the input datasets is a list, it is set 
to 8 as a default. If there are less than 8 samples in the input, it is set 
to half of the number of samples as a default.}

\item{method}{A character string indicating which correlation coefficient is
to be computed. One of "pearson" (default), "kendall", or "spearman": can be
abbreviated.}

\item{maxFrom}{Select whether to display the maximum value from dataset's PCs
or avgLoadings. Under the default (\code{maxFrom="PC"}), the maximum
correlation coefficient from top 8 PCs for each avgLoading will be selected
as an output. If you choose (\code{maxFrom="avgLoading"}), the avgLoading
with the maximum correlation coefficient with each PC will be in the output.}

\item{level}{Output format of validated result. Two options are available:
\code{c("max", "all")}. Default is "max", which outputs the matrix containing
only the maximum coefficient. To get the coefficient of all PCs, set this
argument as "all". \code{level = "all"} can be used only for one dataset.}

\item{scale}{Default is \code{FALSE}. If it is set to \code{TRUE}, dataset
will be row normalized.}
}
\value{
A data frame containing the maximum pearson correlation coefficient
between the top `n` PCs of the dataset and pre-calculated average loadings
(in row) of training datasets (\code{score} column). It also contains other
metadata associated with each RAV: \code{PC} for one of the top `n` PCs of 
the dataset that results in the given \code{score}, \code{sw} for the average
silhouette width of the RAV, \code{cl_size} for the size of each RAV.

If the input for \code{dataset} argument is a list of different datasets,
each row of the output represents a new dataset for test, and each column
represents clusters from training datasets. If \code{level = "all"}, a list
containing the matrices of the pearson correlation coefficient between all
top 8 PCs of the datasets and avgLoading.
}
\description{
Validate new datasets
}
\examples{
data(miniRAVmodel)
library(bcellViper)
data(bcellViper)
validate(dset, miniRAVmodel)
validate(dset, miniRAVmodel, maxFrom = "avgLoading")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extractPC.R
\name{extractPC}
\alias{extractPC}
\title{PCA on gene expression profile}
\usage{
extractPC(x)
}
\arguments{
\item{x}{a numeric or complex matrix (or data frame) which provides the gene
expression data for the principal components analysis. Genes in the rows and
samples in the columns.}
}
\value{
A \code{\link[stats]{prcomp}} object.
}
\description{
Performs a principal components analysis on the given data matrix and returns
the results as an object of class \code{\link[stats]{prcomp}}.
}
\examples{
m = matrix(rnorm(100),ncol=5)
extractPC(m)

}
\seealso{
\code{\link[stats]{prcomp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GenomicSignatures-class.R
\docType{class}
\name{PCAGenomicSignatures-class}
\alias{PCAGenomicSignatures-class}
\alias{.PCAGenomicSignatures}
\title{PCAGenomicSignatures-class}
\arguments{
\item{x}{A \code{GenomicSignatures-class} object}

\item{value}{See details.}
}
\description{
PCA-based \code{\link{GenomicSignatures-class}}.
}
\details{
PCAGenomicSignatures
}
\section{Slots}{

\describe{
\item{\code{trainingData}}{A \code{\link[S4Vectors]{DataFrame}} class object for
metadata associated with training data}
}}

\examples{
data(miniRAVmodel)
miniRAVmodel

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drawWordcloud.R
\name{PCinRAV}
\alias{PCinRAV}
\title{Extract the list of PCs in a cluster}
\usage{
PCinRAV(RAVmodel, ind)
}
\arguments{
\item{RAVmodel}{A PCAGenomicSignatures object}

\item{ind}{An index of RAV}
}
\value{
A character vector of PC/study names
}
\description{
A RAV model contain clusters of PCs from individual
studies. This function extracts the names of the original
PCs from the RAV model given the index in the RAV model.
}
\examples{
data(miniRAVmodel)
PCinRAV(miniRAVmodel,695)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotAnnotatedPCA.R
\name{plotAnnotatedPCA}
\alias{plotAnnotatedPCA}
\title{Two-dimensional PCA plot with the PC annotation}
\usage{
plotAnnotatedPCA(
  dataset,
  RAVmodel,
  PCnum,
  val_all = NULL,
  scoreCutoff = 0.5,
  nesCutoff = NULL,
  color_by = NULL,
  color_lab = NULL,
  trimed_pathway_len = 45
)
}
\arguments{
\item{dataset}{A gene expression profile to be validated. Different classes
of objects can be used including ExpressionSet, SummarizedExperiment,
RangedSummarizedExperiment, or matrix. Rownames (genes) should be in symbol
format. If it is a matrix, genes should be in rows and samples in columns.}

\item{RAVmodel}{PCAGenomicSignatures-class object}

\item{PCnum}{A numeric vector length of 2. The values should be 
between 1 and 8.}

\item{val_all}{The output from \code{\link{validate}}}

\item{scoreCutoff}{A numeric value for the minimum correlation.
Default 0.5.}

\item{nesCutoff}{A numeric value for the minimum NES. Default is \code{NULL}
and the suggested value is 3.}

\item{color_by}{A named vector with the feature you want to color by. Name
should be match with the sample names of the dataset.}

\item{color_lab}{A name for color legend. If this argument is not provided,
the color legend will be labeled as "Color By" by default.}

\item{trimed_pathway_len}{Positive inter values, which is the display width
of pathway names. Default is 45.}
}
\value{
Scatter plot and the table with annotation. If enriched pathway
didn't pass the \code{scoreCutoff} the table will be labeled as "No
significant pathways". If any enriched pathway didn't pass the
\code{nesCutoff}, it will labeled as NA.
}
\description{
Two-dimensional PCA plot with the PC annotation
}
\examples{
data(miniRAVmodel)
library(bcellViper)
data(bcellViper)
\dontrun{
plotAnnotatedPCA(exprs(dset), miniRAVmodel, PCnum = c(1,2))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/buildAvgLoading.R
\name{.calculateSilhouetteWidth}
\alias{.calculateSilhouetteWidth}
\title{Calculate Silhouette Information of RAVs}
\usage{
.calculateSilhouetteWidth(dat, kmeansRes)
}
\arguments{
\item{dat}{A matrix with all the top PCs from training data to be clustered.}

\item{kmeansRes}{Output from \code{stats::kmeans}.}
}
\value{
Silhouette-class object, which is an n x 3 matrix with attributes.
}
\description{
The silhouette value is a measure of how similar an object is
to its own cluster (cohesion) compared to other clusters (separation). The
silhouette width ranges from -1 to +1, where a high value indicates that
the object is well matched to its own cluster and poorly matched to
neighboring clusters.
}
\seealso{
\code{\link[stats]{kmeans}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PCAGenomicSignatures-methods.R
\name{.RAVName}
\alias{.RAVName}
\title{Formatting RAV name}
\usage{
.RAVName(x, ...)
}
\arguments{
\item{x}{PCAGenomicSignatures object}

\item{...}{Additional arguments for supporting functions.}
}
\value{
a character vector
}
\description{
Keep the name with 'k + cluster number + number of PCs + number
of unique studies' info during the model construction to make it easy to keep
track of them, but at the PCAGenomicSignatures-class object building step,
covert them into 'RAV + cluster number'.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{filterList}
\alias{filterList}
\title{RAVs that will output with quality-control messages}
\format{
A named list with four elements - "Cluster_Size_filter",
"GSEA_C2_filter", "GSEA_PLIERpriors_filter", and "Redundancy_filter".
}
\usage{
filterList
}
\description{
RAVs that will output with quality-control messages
}
\author{
Sehyun Oh \email{shbrief@gmail.com}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{miniRAVmodel}
\alias{miniRAVmodel}
\title{RAVmodel from 536 studies, annotated with MSigDB C2}
\format{
PCAGenomicSignatures
}
\usage{
miniRAVmodel
}
\description{
A object providing a miniature version of RAVmodel_C2 (PCAGenomicSignatures
object constructed from 536 studies and annotated with MSigDB C2).
}
\author{
Sehyun Oh \email{shbrief@gmail.com}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotateRAV.R
\name{annotateRAV}
\alias{annotateRAV}
\title{Search the top enriched pathways for RAV}
\usage{
annotateRAV(RAVmodel, ind, n = 5, abs = FALSE)
}
\arguments{
\item{RAVmodel}{PCAGenomicSignatures object.}

\item{ind}{An integer for RAV you want to check the enriched pathways.}

\item{n}{A number of top enriched pathways to output. Default is 5.}

\item{abs}{Default is \code{FALSE}. If it's set to \code{TRUE}, the enriched
pathways will be listed based on \code{abs(NES)}.}
}
\value{
A data frame with \code{n} rows and 4 columns;
Description, NES, pvalue, and qvalues
}
\description{
Search the top enriched pathways for RAV
}
\examples{
data(miniRAVmodel)
annotateRAV(miniRAVmodel, ind = 695)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateScore.R
\name{calculateScore}
\alias{calculateScore}
\title{Calculate the validation score for a new dataset}
\usage{
calculateScore(dataset, RAVmodel, rescale.after = TRUE)
}
\arguments{
\item{dataset}{A gene expression profile to be validated. Different classes
of objects can be used including ExpressionSet, SummarizedExperiment,
RangedSummarizedExperiment, or matrix. Rownames (genes) should be in symbol
format. If it is a matrix, genes should be in rows and samples in columns.}

\item{RAVmodel}{PCAGenomicSignatures object. A matrix of average loadings, an
output from \code{buildAvgLoading}, can be directly provided.}

\item{rescale.after}{Under the default (\code{TRUE}), the continuous scores
are rescaled post assignment, so average loadings have the same standard
deviation in different studies. If it is \code{FALSE}, the rescaling of
column (= dividing by \code{sqrt(sum(x^2}) is done before score assignment.}
}
\value{
A list containing the score matrices for input datasets. Scores are
assigned to each sample (row) for each cluster (column).
}
\description{
Calculate the validation score for a new dataset
}
\examples{
data(miniRAVmodel)
library(bcellViper)
data(bcellViper)
score <- calculateScore(dset, miniRAVmodel)

data(miniTCGA)
score <- calculateScore(miniTCGA, miniRAVmodel)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableRAVmodel.R
\name{availableRAVmodel}
\alias{availableRAVmodel}
\title{List the available RAVmodels}
\usage{
availableRAVmodel(simplify = TRUE)
}
\arguments{
\item{simplify}{Default is \code{TRUE}. If it is set to \code{FALSE}, the
additional metadata of different versions of RAVmodel}
}
\value{
Under the default, this function will return a data frame with four
columns - prior, version, update, pkg_version. 
\itemize{
    \item \code{prior} : Different gene sets used for RAVmodel annotation.
    Currently, two are available - \code{C2} for MSigDB C2 (curated gene 
    sets), and \code{PLIERpriors} for bloodCellMarkersIRISDMAP, svmMarkers, 
    and canonicalPathways
    \item \code{version} : RAVmodel's version, which can be an input for 
    \code{version} argument of \code{\link{getModel}} function
    \item \code{update} : Date the RAVmodel is updated
    \item \code{pkg_version} : Compatible version of GenomicSuperSignature
}
}
\description{
List the available RAVmodels
}
\examples{
availableRAVmodel()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampleScoreHeatmap.R
\name{sampleScoreHeatmap}
\alias{sampleScoreHeatmap}
\title{Plot heatmap of the sample scores}
\usage{
sampleScoreHeatmap(
  score,
  dataName,
  modelName,
  cluster_rows = TRUE,
  cluster_columns = TRUE,
  show_row_names = TRUE,
  show_column_names = TRUE,
  row_names_gp = 0.7,
  column_names_gp = 5,
  ...
)
}
\arguments{
\item{score}{An output from \code{\link{calculateScore}} function, which is
a matrix with samples (row) and PrcompClusters (column) If it is a simple
vector, it will be converted to a one-column matrix.}

\item{dataName}{Title on the row. The name of the dataset to be scored.}

\item{modelName}{Title on the column. The RAVmodel used for scoring.}

\item{cluster_rows}{A logical. Under the default (\code{TRUE}), rows will be
clustered.}

\item{cluster_columns}{A logical. Under the default (\code{TRUE}), columns
will be clustered.}

\item{show_row_names}{Whether show row names. Default is \code{TRUE}, showing
the row name.}

\item{show_column_names}{Whether show column names. Default is \code{TRUE},
showing the column name.}

\item{row_names_gp}{Graphic parameters for row names. The default is 0.7.}

\item{column_names_gp}{Graphic parameters for column names. The default is 5.}

\item{...}{Any additional argument for \code{\link[ComplexHeatmap]{Heatmap}}}
}
\value{
A heatmap of the sample score. Rows represent samples and columns
represent RAVs.
}
\description{
Plot heatmap of the sample scores
}
\examples{
data(miniRAVmodel)
library(bcellViper)
data(bcellViper)
score <- calculateScore(dset, miniRAVmodel)
sampleScoreHeatmap(score, dataName="bcellViper", modelName="miniRAVmodel")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GenomicSignatures-methods.R
\name{GenomicSignatures-methods}
\alias{GenomicSignatures-methods}
\alias{RAVindex}
\alias{colData}
\alias{metadata}
\alias{geneSets}
\alias{updateNote}
\alias{geneSets<-}
\alias{updateNote<-}
\alias{RAVindex,GenomicSignatures-method}
\alias{geneSets,GenomicSignatures-method}
\alias{updateNote,GenomicSignatures-method}
\alias{version,GenomicSignatures-method}
\alias{geneSets<-,GenomicSignatures-method}
\alias{updateNote<-,GenomicSignatures-method}
\title{Methods and accesors for \code{GenomicSignatures} object}
\usage{
\S4method{RAVindex}{GenomicSignatures}(x)

\S4method{geneSets}{GenomicSignatures}(x)

\S4method{updateNote}{GenomicSignatures}(x)

\S4method{version}{GenomicSignatures}(x)

\S4method{geneSets}{GenomicSignatures}(x) <- value

\S4method{updateNote}{GenomicSignatures}(x) <- value
}
\arguments{
\item{x}{A \code{GenomicSignatures} object}

\item{value}{See details.}
}
\value{
A GenomicSignatures object for the constructor
}
\description{
The default contents of \code{GenomicSignatures} object, with
a set of getter and setter generic functions, which extract either the
\code{assay}, \code{colData}, or \code{metadata} slots of a
\code{\link{GenomicSignatures-class}} object. When you create this object,
\code{colData$studies} should be populated before adding any information in
\code{trainingData} slot.
}
\details{
\itemize{
    \item assay(x) : RAVindex (= avgLoadings) containing genes x RAVs
    \item metadata(x) : Metadata associated with RAVindex building process
    \item colData(x) : Information on RAVs
}
}
\section{Setters}{

Setter method values (i.e., \code{function(x) <- value}):
\itemize{
    \item metadata<- : Assign metadata
    \item coldata<- : Assign extra information associated with RAVs
    \item geneSets<- : A character vector containing the name of gene sets
    used to annotate average loadings
    \item updateNote<- : A character vector. Describes the main feature of a
    model construction
}
}

\section{Getters}{

\itemize{
   \item RAVindex : Equivalent to \code{assays(x)$RAVindex}
   \item geneSets : Access the \code{metadata(x)$geneSets} slot
   \item updateNote : Access the \code{metadata(x)$updateNote} slot
   \item version : Access the \code{metadata(x)$version} slot
}
}

\examples{
data(miniRAVmodel)
miniRAVmodel


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GenomicSignatures-class.R
\docType{class}
\name{GenomicSignatures-class}
\alias{GenomicSignatures-class}
\title{Virtual class inherited from SummarizedExperiment}
\arguments{
\item{x}{A \code{GenomicSignatures-class} object}

\item{value}{See details.}
}
\description{
GenomicSignatures is a virtual class inherited from
SummarizedExperiment and hosts GenomicSignatures models built from different
dimensional reduction methods. Currently, PCA-based model, called
PCAGenomicSignatures, is available.
}
\details{
GenomicSignatures
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validatedSignatures.R
\name{validatedSignatures}
\alias{validatedSignatures}
\title{Validation result in data frame}
\usage{
validatedSignatures(
  val_all,
  RAVmodel,
  num.out = 5,
  scoreCutoff = NULL,
  swCutoff = NULL,
  clsizeCutoff = NULL,
  indexOnly = FALSE,
  whichPC = NULL,
  filterMessage = TRUE
)
}
\arguments{
\item{val_all}{An output matrix from \code{\link{validate}} function. If this
input is from multiple datasets, only \code{scoreCutoff} argument will be
considered and other inputs will be ignored.}

\item{RAVmodel}{PCAGenomicSignatures-class object. RAVmodel used to prepare
\code{val_all} input.}

\item{num.out}{A number of highly validated RAVs to output. Default is 5.
If any of the cutoff parameters are provided, \code{num.out} or the number of
filtered RAVs, whichever smaller, will be chosen.}

\item{scoreCutoff}{A numeric value for the minimum correlation. For
multi-studies case, the default is 0.7.}

\item{swCutoff}{A numeric value for the minimum average silhouette width.}

\item{clsizeCutoff}{An integer value for the minimum cluster size.}

\item{indexOnly}{A logical. Under the default (= \code{FALSE}), the detailed
information on validated RAVs, such as score, average silhouette width,
cluster size, is printed. If it is set TRUE, only the RAV number will be
printed.}

\item{whichPC}{An integer value between 1 and 8. PC number of your data to
check the validated signatures with. Under the default (\code{NULL}), it
outputs top scored signatures with any PC of your data.}

\item{filterMessage}{A logical. Under the default \code{TRUE}, any output
RAV belong to the filtering list will give a message. Silence this message
with \code{filterMessage=FALSE}. You can check the filter list using
\code{data("filterList")}.}
}
\value{
A subset of the input matrix, which meets the given condition.
}
\description{
Validation result in data frame
}
\examples{
data(miniRAVmodel)
library(bcellViper)
data(bcellViper)
val_all <- validate(dset, miniRAVmodel)
validatedSignatures(val_all, miniRAVmodel, num.out = 3, scoreCutoff = 0)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmNaInf.R
\name{rmNaInf}
\alias{rmNaInf}
\title{Remove rows with missing and Inf values from a matrix}
\usage{
rmNaInf(x)
}
\arguments{
\item{x}{A numeric matrix.}
}
\value{
The updated input matrix where rows with
   NA and Inf values are removed.
}
\description{
Remove rows with missing and Inf values from a matrix
}
\examples{
m = matrix(rnorm(100),ncol=10)
m[1,1] = NA

m1 = rmNaInf(m)
dim(m1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getModel.R
\name{getModel}
\alias{getModel}
\title{Download a PCAGenomicSignatures model}
\usage{
getModel(prior = c("C2", "PLIERpriors"), version = "latest", load = TRUE)
}
\arguments{
\item{prior}{The name of gene sets used to annotate PCAGenomicSignatures.
Currently  there are two available options.
\itemize{
    \item \code{C2} : MSigDB C2 (curated gene sets)
    \item \code{PLIERpriors} : bloodCellMarkersIRISDMAP, svmMarkers, and
    canonicalPathways
}}

\item{version}{Default is \code{latest}. Available versions are listed in
\code{version} column of \code{\link{[GenomicSuperSignature]availableRAVmodel()}} output.}

\item{load}{Default is \code{TRUE}. If it's set to \code{FALSE}, the models
are just downloaded to cache but not loaded into memory.}
}
\value{
File cache location or PCAGenomicSignatures object loaded from it.
}
\description{
Download a PCAGenomicSignatures model
}
\examples{
z = getModel("C2")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMetadata.R
\name{getStudyInfo}
\alias{getStudyInfo}
\title{Extract information on a specific training dataset}
\usage{
getStudyInfo(RAVmodel, study)
}
\arguments{
\item{RAVmodel}{A PCAGenomicSignatures object}

\item{study}{A character for SRA study accession.}
}
\value{
A list with three elements: studyTitle, studySize (the number of
samples from this study used in the RAVmodel building), and RAVs. 'RAVs' is
a data frame with three columns - PC (1 to 20), RAV (RAV that the given PC
belongs to), and Variance explained (%). In the example, we used
miniRAVmodel, which doesn't have all the PCA summary information, so the
example will return only the two PCs of the study instead of all twenty.
}
\description{
Extract information on a specific training dataset
}
\examples{
data(miniRAVmodel)
getStudyInfo(miniRAVmodel, "SRP028155")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{res_hcut}
\alias{res_hcut}
\title{Subset of allZ matrix constructed from 8 CRC training datasets}
\format{
\code{hclust} object from \code{factoextra::hcut} function.
}
\usage{
res_hcut
}
\description{
Eight colorectal cancer microarray datasets were used to build RAVmodel and
the intermediate file containing genes and top PCs from each dataset is named
as \code{allZ}. Hierarchical clustering result of \code{allZ} is saved as
\code{res_hcut}.
}
\author{
Sehyun Oh \email{shbrief@gmail.com}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsetEnrichedPathways.R
\name{subsetEnrichedPathways}
\alias{subsetEnrichedPathways}
\title{Subset enriched pathways of RAV}
\usage{
subsetEnrichedPathways(
  RAVmodel,
  ind = NULL,
  n = 10,
  both = FALSE,
  include_nes = FALSE
)
}
\arguments{
\item{RAVmodel}{PCAGenomicSignatures object. Also an output from
\code{\link[clusterProfiler]{GSEA}} can be used.}

\item{ind}{A numeric vector containing the RAV number you want to check
enriched pathways. If not specified, this function returns results from all
the RAVs.}

\item{n}{The number of top and bottom pathways to be selected based on
normalized enrichment score (NES).}

\item{both}{Default is \code{FALSE}, where only the top \code{n} pathways
will be printed. If it is set to \code{TRUE}, the output will contain both
top and bottom \code{n} pathways.}

\item{include_nes}{Defalt is \code{FALSE}. If it set to \code{TRUE}, the
output will include both description and NES of the enriched pathway.}
}
\value{
A DataFrame with top and bottom \code{n} pathways from the
enrichment results.
}
\description{
Subset enriched pathways of RAV
}
\examples{
data(miniRAVmodel)

# all RAVS in model
subsetEnrichedPathways(miniRAVmodel,n=5)

# only a specific RAV (note the colnames above)
subsetEnrichedPathways(miniRAVmodel,ind=695,n=5)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate.R
\name{.loadingCor}
\alias{.loadingCor}
\title{Validating new dataset}
\usage{
.loadingCor(dataset, avgLoading, n = n, method = method, scale = FALSE)
}
\arguments{
\item{dataset}{A gene expression profile to be validated. Different classes
of objects can be used including ExpressionSet, SummarizedExperiment,
RangedSummarizedExperiment, or matrix. Rownames (genes) should be in 
human gene symbol format (HGNC). If dataset is a matrix, genes should be in 
rows and samples in columns. RNA-seq counts should be log(count + 1) 
prior to the `validate()` call.}

\item{avgLoading}{A matrix with genes by RAVs.}

\item{n}{A integer. The number of PCs to use for validation. It should be
equal or less then the number of samples in the input dataset.}

\item{method}{A character string indicating which correlation coefficient is
to be computed. One of "pearson" (default), "kendall", or "spearman": can be
abbreviated.}

\item{scale}{Default is \code{FALSE}. If it is set to \code{TRUE}, rows 
will be converted to z-score prior to PCA.}
}
\value{
A matrix of Pearson correlation coefficient (default, defined through
\code{method} argument) between RAVs (row) and the top 8 PCs from the
datasets (column)
}
\description{
Validating new dataset
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMetadata.R
\name{getRAVInfo}
\alias{getRAVInfo}
\title{Extract information on a specific RAV}
\usage{
getRAVInfo(RAVmodel, ind)
}
\arguments{
\item{RAVmodel}{A PCAGenomicSignatures object}

\item{ind}{An index of RAV}
}
\value{
A list with four elements: clusterSize, silhouetteWidth,
enrichedPathways (the number of enriched pathways), and members.
The 'members' is the summary table of PCs in RAV, containing three columns:
studyName, PC, and Variance explained (%).
}
\description{
Extract information on a specific RAV
}
\examples{
data(miniRAVmodel)
getRAVInfo(miniRAVmodel, ind = 438)

}
